@@ -11,7 +11,7 @@
  * This file contains all the "glue code" required to use the canon
  * driver with libgphoto2.
  *
- * $Id: library.c,v 1.93.2.1 2005/06/07 17:45:17 hfiguiere Exp $
+ * $Id: library.c 8679 2006-03-31 17:42:09Z marcusmeissner $
  *
  ****************************************************************************/
 
@@ -98,7 +98,7 @@ extern long int timezone;
  */
 int camera_id (CameraText *id)
 {
-	GP_DEBUG ("camera_id()");
+	/* GP_DEBUG ("camera_id()"); */
 
 	strcpy (id->text, "canon");
 
@@ -157,7 +157,7 @@ camera_abilities (CameraAbilitiesList *list)
 	int i;
 	CameraAbilities a;
 
-	GP_DEBUG ("camera_abilities()");
+	/* GP_DEBUG ("camera_abilities()"); */
 
 	for (i = 0; models[i].id_str; i++) {
 		memset (&a, 0, sizeof (a));
@@ -565,7 +565,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 				GP_DEBUG (_("%s is a file type for which no thumbnail is provided"),canon_path);
 				return (GP_ERROR_NOT_SUPPORTED);
 			}
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 			/* Check if we have libexif, it is a jpeg file
 			 * and it is not a PowerShot Pro 70 (which
 			 * does not support EXIF), return not
@@ -579,7 +579,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 					return (GP_ERROR_NOT_SUPPORTED);
 				}
 			}
-#endif /* HAVE_EXIF */
+#endif /* HAVE_LIBEXIF */
 			if (*thumbname == '\0') {
 				/* file internal thumbnail */
 				ret = canon_int_get_thumbnail (camera, canon_path, &data,
@@ -592,7 +592,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			break;
 
 		case GP_FILE_TYPE_EXIF:
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 			/* the PowerShot Pro 70 does not support EXIF */
 			if (camera->pl->md->model == CANON_CLASS_2)
 				return (GP_ERROR_NOT_SUPPORTED);
@@ -616,7 +616,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 #else
 			GP_DEBUG ("get_file_func: EXIF file type requested but no libexif");
 			return (GP_ERROR_NOT_SUPPORTED);
-#endif /* HAVE_EXIF */
+#endif /* HAVE_LIBEXIF */
 			break;
 
 		default:
@@ -687,7 +687,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			gp_file_set_data_and_size (file, data, datalen);
 			gp_file_set_name (file, filename);
 			break;
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 		case GP_FILE_TYPE_EXIF:
 			if ( !is_cr2 ( filename ) )
 				gp_file_set_mime_type (file, GP_MIME_JPEG);
@@ -695,7 +695,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 				gp_file_set_mime_type (file, GP_MIME_EXIF);
 			gp_file_set_data_and_size (file, data, datalen);
 			break;
-#endif /* HAVE_EXIF */
+#endif /* HAVE_LIBEXIF */
 		default:
 			/* this case should've been caught above anyway */
 			if (data)
@@ -1520,6 +1520,17 @@ remove_dir_func (CameraFilesystem *fs, const char *folder, const char *name, voi
  * Returns: gphoto2 error code
  *
  */
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.folder_list_func = folder_list_func,
+	.get_info_func = get_info_func,
+	.get_file_func = get_file_func,
+	.del_file_func = delete_file_func,
+	.put_file_func = put_file_func,
+	.make_dir_func = make_dir_func,
+	.remove_dir_func = remove_dir_func
+};
+
 int
 camera_init (Camera *camera, GPContext *context)
 {
@@ -1538,18 +1549,7 @@ camera_init (Camera *camera, GPContext *context)
 	camera->functions->about = camera_about;
 
 	/* Set up the CameraFilesystem */
-	gp_filesystem_set_list_funcs (camera->fs, file_list_func, folder_list_func, camera);
-	gp_filesystem_set_info_funcs (camera->fs, get_info_func, NULL, camera);
-	gp_filesystem_set_file_funcs (camera->fs, get_file_func, delete_file_func, camera);
-	gp_filesystem_set_folder_funcs (camera->fs,
-#ifdef CANON_EXPERIMENTAL_UPLOAD
-					put_file_func, 
-#else
-					(camera->port->type == GP_PORT_SERIAL)?put_file_func:NULL,
-#endif /* CANON_EXPERIMENTAL_UPLOAD */
-					NULL, make_dir_func,
-					remove_dir_func, camera);
-
+	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 	camera->pl = malloc (sizeof (CameraPrivateLibrary));
 	if (!camera->pl)
 		return (GP_ERROR_NO_MEMORY);
