@@ -2,7 +2,7 @@
  *
  * File: usb.h
  *
- * $Id: usb.h,v 1.41.2.3 2005/06/07 17:45:19 hfiguiere Exp $
+ * $Id: usb.h 8062 2005-06-11 15:57:53Z hun $
  *
  ****************************************************************************/
 
@@ -280,7 +280,7 @@ int canon_usb_set_file_attributes (Camera *camera,
 				   unsigned int attr_bits,
 				   const char *dir, const char *file,
 				   GPContext *context);
-int canon_usb_ready (Camera *camera);
+int canon_usb_ready (Camera *camera, GPContext *context);
 int canon_usb_identify (Camera *camera, GPContext *context);
 
 #endif /* _CANON_USB_H */
