@@ -206,7 +206,7 @@ static int file_list_func(CameraFilesystem *fs, const char *folder,
 }
 
 #define GFD_BUFSIZE 19432
-static char *digita_file_get(Camera *camera, const char *folder, 
+static unsigned char *digita_file_get(Camera *camera, const char *folder, 
 			     const char *filename, int thumbnail, int *size,
 			     GPContext *context)
 {
@@ -341,7 +341,7 @@ static int get_file_func(CameraFilesystem *fs, const char *folder,
 
 	switch (type) {
 	case GP_FILE_TYPE_NORMAL:
-		gp_file_set_data_and_size(file, data, buflen);
+		gp_file_set_data_and_size(file, (char*)data, buflen);
 		gp_file_set_mime_type(file, GP_MIME_JPEG);
 		break;
 	case GP_FILE_TYPE_PREVIEW:
@@ -367,7 +367,7 @@ static int get_file_func(CameraFilesystem *fs, const char *folder,
 
 		strcpy(ppm, ppmhead);
 
-		decode_ycc422(data + 16, width, height, ppm + strlen(ppmhead));
+		decode_ycc422(data + 16, width, height, (unsigned char*)(ppm + strlen(ppmhead)));
 		free(data);
 
 		gp_file_set_mime_type(file, GP_MIME_PPM);
@@ -433,6 +433,13 @@ static int delete_file_func(CameraFilesystem *fs, const char *folder,
         return digita_file_delete(camera, folder, filename, context);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.folder_list_func = folder_list_func,
+	.get_file_func = get_file_func,
+	.del_file_func = delete_file_func
+};
+
 int camera_init(Camera *camera, GPContext *context)
 {
 	int ret = 0;
@@ -446,10 +453,7 @@ int camera_init(Camera *camera, GPContext *context)
 	camera->functions->about	= camera_about;
 
 	/* Set up the CameraFilesystem */
-	gp_filesystem_set_list_funcs(camera->fs, file_list_func,
-				      folder_list_func, camera);
-	gp_filesystem_set_file_funcs(camera->fs, get_file_func,
-				      delete_file_func, camera);
+	gp_filesystem_set_funcs(camera->fs, &fsfuncs, camera);
 
 	GP_DEBUG( "Initializing the camera");
 
