@@ -1,6 +1,7 @@
 /* directory.c
  *
  * Copyright  2001 Lutz Mller <lutz@users.sf.net>
+ * Copyright  2005 Marcus Meissner <marcus@jet.franken.de>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,7 +28,7 @@
 #include <utime.h>
 #include <unistd.h>
 
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 #include <libexif/exif-data.h>
 #endif
 
@@ -64,8 +65,18 @@ static const struct {
 	{"tif",  GP_MIME_TIFF},
 	{"ppm",  GP_MIME_PPM},
 	{"pgm",  GP_MIME_PGM},
+	{"avi",  GP_MIME_AVI},
+	{"mov",  GP_MIME_QUICKTIME},
+	{"mpg",  GP_MIME_MPEG},
 	{"pbm", "image/x-portable-bitmap"},
+	{"crw",  GP_MIME_CRW},
+	{"cr2",  GP_MIME_RAW},
+	{"nef",  GP_MIME_RAW},
+	{"mrw",  GP_MIME_RAW},
+	{"dng",  GP_MIME_RAW},
+	{"sr2",  GP_MIME_RAW},
 	{"png",  GP_MIME_PNG},
+	{"wav",  GP_MIME_WAV},
 	{NULL, NULL}
 };
 
@@ -111,15 +122,11 @@ int camera_abilities (CameraAbilitiesList *list)
 	memset(&a, 0, sizeof(a));
         strcpy(a.model, "Directory Browse");
 	a.status = GP_DRIVER_STATUS_PRODUCTION;
-        a.port     = GP_PORT_NONE;
+        a.port     = GP_PORT_DISK;
         a.speed[0] = 0;
 
-#ifdef DEBUG
-	a.operations = GP_OPERATION_CONFIG | GP_OPERATION_CAPTURE_PREVIEW |
-		       GP_OPERATION_CAPTURE_IMAGE;
-#else
-        a.operations = GP_OPERATION_CONFIG;
-#endif
+        a.operations = GP_OPERATION_NONE;
+
 #ifdef DEBUG
         a.file_operations = GP_FILE_OPERATION_PREVIEW |
 			    GP_FILE_OPERATION_DELETE | 
@@ -134,6 +141,11 @@ int camera_abilities (CameraAbilitiesList *list)
 
         gp_abilities_list_append(list, a);
 
+	/* Since "Directory Browse" is hardcoded in clients,
+	 * better also add a new name here.
+	 */
+        strcpy(a.model, "Mass Storage Camera");
+        gp_abilities_list_append(list, a);
         return (GP_OK);
 }
 
@@ -141,53 +153,71 @@ static int
 file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		void *data, GPContext *context)
 {
-	GP_SYSTEM_DIR dir;
-	GP_SYSTEM_DIRENT de;
+	gp_system_dir dir;
+	gp_system_dirent de;
 	char buf[1024], f[1024];
 	unsigned int id, n;
+	Camera *camera = (Camera*)data;
 
-	dir = GP_SYSTEM_OPENDIR ((char*) folder);
+	if (camera->port->type == GP_PORT_DISK) {
+		GPPortSettings settings;
+
+		gp_port_get_settings (camera->port, &settings);
+		snprintf (f, sizeof(f), "%s/%s/",
+			settings.disk.mountpoint, 
+			folder
+		);
+		/* UNIX / is empty, or we recurse through the whole fs */
+		if (	(!strcmp(settings.disk.mountpoint, "") ||
+			 !strcmp(settings.disk.mountpoint, "/")) &&
+			!strcmp(folder,"/")
+		)
+			return GP_OK;
+	} else {
+		/* old style access */
+		if (folder[strlen(folder)-1] != '/')
+			sprintf (f, "%s%c", folder, '/');
+		else
+			strcpy (f, folder);
+	}
+	dir = gp_system_opendir ((char*) f);
 	if (!dir)
 		return (GP_ERROR);
 	
 	/* Make sure we have 1 delimiter */
-	if (folder[strlen(folder)-1] != '/')
-		sprintf (f, "%s%c", folder, '/');
-	else
-		strcpy (f, folder);
 
 	/* Count the files */
 	n = 0;
-	while (GP_SYSTEM_READDIR (dir))
+	while (gp_system_readdir (dir))
 		n++;
 
-	GP_SYSTEM_CLOSEDIR (dir);
-	dir = GP_SYSTEM_OPENDIR (folder);
+	gp_system_closedir (dir);
+	dir = gp_system_opendir (f);
 	if (!dir)
 		return (GP_ERROR);
 	id = gp_context_progress_start (context, n, _("Listing files in "
-				"'%s'..."), folder);
+				"'%s'..."), f);
 	n = 0;
-	while ((de = GP_SYSTEM_READDIR(dir))) {
+	while ((de = gp_system_readdir(dir))) {
 
 		/* Give some feedback */
 		gp_context_progress_update (context, id, n + 1);
 		gp_context_idle (context);
 		if (gp_context_cancel (context) == GP_CONTEXT_FEEDBACK_CANCEL) {
-			GP_SYSTEM_CLOSEDIR (dir);
+			gp_system_closedir (dir);
 			return (GP_ERROR_CANCEL);
 		}
 
-		if (strcmp(GP_SYSTEM_FILENAME(de), "." ) &&
-		    strcmp(GP_SYSTEM_FILENAME(de), "..")) {
-			sprintf (buf, "%s%s", f, GP_SYSTEM_FILENAME (de));
-			if (GP_SYSTEM_IS_FILE (buf) && (get_mime_type (buf)))
-				gp_list_append (list, GP_SYSTEM_FILENAME (de),
+		if (strcmp(gp_system_filename(de), "." ) &&
+		    strcmp(gp_system_filename(de), "..")) {
+			sprintf (buf, "%s%s", f, gp_system_filename (de));
+			if (gp_system_is_file (buf) && (get_mime_type (buf)))
+				gp_list_append (list, gp_system_filename (de),
 						NULL);
 		}
 		n++;
 	}
-	GP_SYSTEM_CLOSEDIR (dir);
+	gp_system_closedir (dir);
 	gp_context_progress_stop (context, id);
 
 	return (GP_OK);
@@ -197,90 +227,77 @@ static int
 folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		  void *data, GPContext *context)
 {
-	GP_SYSTEM_DIR dir;
-	GP_SYSTEM_DIRENT de;
+	gp_system_dir dir;
+	gp_system_dirent de;
 	char buf[1024], f[1024];
-#ifdef FOLLOW_LINKS
-	char link[1024];
-#endif
-	const char *dirname;
-	int view_hidden=1;
 	unsigned int id, n;
 	struct stat st;
+	Camera *camera = (Camera*)data;
 
-	if (gp_setting_get ("directory", "hidden", buf) == GP_OK)
-		view_hidden = atoi (buf);
+	if (camera->port->type == GP_PORT_DISK) {
+		GPPortSettings settings;
 
-	if (lstat (folder, &st) != 0) {
-		gp_context_error (context, _("Could not get information "
-				  "about '%s' (%m)."), folder);
-		return (GP_ERROR);
+		gp_port_get_settings (camera->port, &settings);
+		snprintf (f, sizeof(f), "%s/%s/",
+			settings.disk.mountpoint, 
+			folder
+		);
+		/* UNIX / is empty, or we recurse through the whole fs */
+		if (	(!strcmp(settings.disk.mountpoint, "") ||
+			 !strcmp(settings.disk.mountpoint, "/")) &&
+			!strcmp(folder,"/")
+		)
+			return GP_OK;
+	} else {
+		/* old style access */
+		/* Make sure we have 1 delimiter */
+		if (folder[strlen(folder)-1] != '/')
+			sprintf (f, "%s%c", folder, '/');
+		else
+			strcpy (f, folder);
 	}
-
-#ifdef FOLLOW_LINKS
-	/* Check if this is a link */
-	if (S_ISLNK (st.st_mode)) {
-		if (readlink (folder, link, sizeof (link) < 0)) {
-			gp_context_error (context, _("Could not follow the "
-				"link '%s' (%m)."), folder);
-			return (GP_ERROR);
-		}
-		GP_DEBUG ("Following link '%s' -> '%s'...", folder, link);
-		return (folder_list_func (fs, link, list, data, context));
-	}
-#endif
-
-	dir = GP_SYSTEM_OPENDIR ((char*) folder);
+	dir = gp_system_opendir ((char*) f);
 	if (!dir)
 		return (GP_ERROR);
-
-	/* Make sure we have 1 delimiter */
-	if (folder[strlen(folder)-1] != '/')
-		sprintf (f, "%s%c", folder, '/');
-	else
-		strcpy (f, folder);
-
 	/* Count the files */
 	n = 0;
-	while (GP_SYSTEM_READDIR (dir))
+	while (gp_system_readdir (dir))
 		n++;
+	gp_system_closedir (dir);
 
-	GP_SYSTEM_CLOSEDIR (dir);
-	dir = GP_SYSTEM_OPENDIR (folder);
+	dir = gp_system_opendir (f);
 	if (!dir)
 		return (GP_ERROR);
 	id = gp_context_progress_start (context, n, _("Listing folders in "
 					"'%s'..."), folder);
 	n = 0;
-	while ((de = GP_SYSTEM_READDIR (dir))) {
-
+	while ((de = gp_system_readdir (dir))) {
 		/* Give some feedback */
 		gp_context_progress_update (context, id, n + 1);
 		gp_context_idle (context);
 		if (gp_context_cancel (context) == GP_CONTEXT_FEEDBACK_CANCEL) {
-			GP_SYSTEM_CLOSEDIR (dir);
+			gp_system_closedir (dir);
 			return (GP_ERROR_CANCEL);
 		}
-		if ((strcmp (GP_SYSTEM_FILENAME (de), "." ) != 0) &&
-		    (strcmp (GP_SYSTEM_FILENAME (de), "..") != 0)) {
-			sprintf (buf, "%s%s", f, GP_SYSTEM_FILENAME (de));
-			dirname = GP_SYSTEM_FILENAME (de);
-			if (GP_SYSTEM_IS_DIR (buf)) {
-				if (dirname[0] != '.')
-					gp_list_append (list,
-							GP_SYSTEM_FILENAME (de),
-							NULL);
-				else if (view_hidden)
-					gp_list_append (list,
-							GP_SYSTEM_FILENAME (de),
-							NULL);
+		if ((strcmp (gp_system_filename (de), "." ) != 0) &&
+		    (strcmp (gp_system_filename (de), "..") != 0)) {
+			sprintf (buf, "%s%s", f, gp_system_filename (de));
+
+			/* lstat ... do not follow symlinks */
+			if (lstat (buf, &st) != 0) {
+				gp_context_error (context, _("Could not get information "
+						  "about '%s' (%m)."), buf);
+				return (GP_ERROR);
 			}
+			if (S_ISDIR (st.st_mode))
+				gp_list_append (list,
+						gp_system_filename (de),
+						NULL);
 		}
 		n++;
 	}
-	GP_SYSTEM_CLOSEDIR (dir);
+	gp_system_closedir (dir);
 	gp_context_progress_stop (context, id);
-
 	return (GP_OK);
 }
 
@@ -289,47 +306,31 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *file,
 		      CameraFileInfo *info, void *data, GPContext *context)
 {
 	char path[1024];
-#ifdef FOLLOW_LINKS
-	char link[1024];
-	char *name;
-#endif
 	const char *mime_type;
 	struct stat st;
+	Camera *camera = (Camera*)data;
 
-	if (strlen (folder) == 1)
-		snprintf (path, sizeof (path), "/%s", file);
-	else
+	if (camera->port->type == GP_PORT_DISK) {
+		GPPortSettings settings;
+
+		gp_port_get_settings (camera->port, &settings);
+		snprintf (path, sizeof(path), "%s/%s/%s",
+			settings.disk.mountpoint, 
+			folder,
+			file
+		);
+	} else {
+		/* old style access */
 		snprintf (path, sizeof (path), "%s/%s", folder, file);
+	}
+
 	if (lstat (path, &st) != 0) {
 		gp_context_error (context, _("Could not get information "
 			"about '%s' in '%s' (%m)."), file, folder);
 		return (GP_ERROR);
 	}
 
-#ifdef FOLLOW_LINKS
-	if (S_ISLNK (st.st_mode)) {
-		if (readlink (path, link, sizeof (link) < 0)) {
-			gp_context_error (context, _("Could not follow the "
-				"link '%s' in '%s' (%m)."), file, folder);
-			return (GP_ERROR);
-		}
-		name = strrchr (link, '/');
-		if (!name)
-			return (GP_ERROR);
-		else {
-			*name = '\0';
-			name++;
-			return (get_info_func (fs, link, name, info, data,
-					       context));
-		}
-	}
-#endif
-
-#ifdef DEBUG
-	info->preview.fields = GP_FILE_INFO_SIZE;
-#else
         info->preview.fields = GP_FILE_INFO_NONE;
-#endif
         info->file.fields = GP_FILE_INFO_SIZE | GP_FILE_INFO_NAME |
                             GP_FILE_INFO_TYPE | GP_FILE_INFO_PERMISSIONS |
 			    GP_FILE_INFO_MTIME;
@@ -342,10 +343,6 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *file,
 		info->file.permissions |= GP_FILE_PERM_DELETE;
         strcpy (info->file.name, file);
         info->file.size = st.st_size;
-#ifdef DEBUG
-	info->preview.size = st.st_size;
-#endif
-
 	mime_type = get_mime_type (file);
 	if (!mime_type)
 		mime_type = "application/octet-stream";
@@ -360,6 +357,21 @@ set_info_func (CameraFilesystem *fs, const char *folder, const char *file,
 {
 	int retval;
 	char path_old[1024], path_new[1024], path[1024];
+	Camera *camera = (Camera*)data;
+
+	if (camera->port->type == GP_PORT_DISK) {
+		GPPortSettings settings;
+
+		gp_port_get_settings (camera->port, &settings);
+		snprintf (path, sizeof(path), "%s/%s/%s",
+			settings.disk.mountpoint, 
+			folder,
+			file
+		);
+	} else {
+		/* old style access */
+		snprintf (path, sizeof (path), "%s/%s", folder, file);
+	}
 
 	/* We don't support updating permissions (yet) */
 	if (info.file.fields & GP_FILE_INFO_PERMISSIONS)
@@ -370,10 +382,6 @@ set_info_func (CameraFilesystem *fs, const char *folder, const char *file,
 
 		utimbuf.actime  = info.file.mtime;
 		utimbuf.modtime = info.file.mtime;
-		if (strlen (folder) == 1)
-			snprintf (path, sizeof (path), "/%s", file);
-		else
-			snprintf (path, sizeof (path), "%s/%s", folder, file);
 		if (utime (path, &utimbuf) != 0) {
 			gp_context_error (context, _("Could not change "
 				"time of file '%s' in '%s' (%m)."),
@@ -432,16 +440,26 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 #ifdef DEBUG
 	unsigned int i, id;
 #endif
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 	ExifData *data;
 	unsigned char *buf;
 	unsigned int buf_len;
-#endif /* HAVE_EXIF */
+#endif /* HAVE_LIBEXIF */
+	Camera *camera = (Camera*)user_data;
 
-	if (strlen (folder) == 1)
-		snprintf (path, sizeof (path), "/%s", filename);
-	else
+	if (camera->port->type == GP_PORT_DISK) {
+		GPPortSettings settings;
+
+		gp_port_get_settings (camera->port, &settings);
+		snprintf (path, sizeof(path), "%s/%s/%s",
+			settings.disk.mountpoint, 
+			folder,
+			filename
+		);
+	} else {
+		/* old style access */
 		snprintf (path, sizeof (path), "%s/%s", folder, filename);
+	}
 
 	switch (type) {
 	case GP_FILE_TYPE_NORMAL:
@@ -452,7 +470,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		result = gp_file_open (file, path);
 		break;
 #endif
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 	case GP_FILE_TYPE_EXIF:
 		data = exif_data_new_from_file (path);
 		if (!data) {
@@ -464,7 +482,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		exif_data_unref (data);
 		gp_file_set_data_and_size (file, buf, buf_len);
 		return (GP_OK);
-#endif /* HAVE_EXIF */
+#endif /* HAVE_LIBEXIF */
 	default:
 		return (GP_ERROR_NOT_SUPPORTED);
 	}
@@ -487,58 +505,6 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	return (GP_OK);
 }
 
-static int
-camera_get_config (Camera *camera, CameraWidget **window, GPContext *context)
-{
-	CameraWidget *widget;
-#ifdef DEBUG
-	CameraWidget *section;
-#endif
-        char buf[256];
-        int val;
-
-        gp_widget_new (GP_WIDGET_WINDOW, _("Directory Browse"), window);
-        gp_widget_new (GP_WIDGET_TOGGLE, _("View hidden directories"),
-                       &widget);
-        gp_setting_get ("directory", "hidden", buf);
-        val = atoi (buf);
-        gp_widget_set_value (widget, &val);
-        gp_widget_append (*window, widget);
-
-#ifdef DEBUG
-	gp_widget_new (GP_WIDGET_SECTION, "Testing", &section);
-	gp_widget_append (*window, section);
-
-	gp_widget_new (GP_WIDGET_TEXT, "Text", &widget);
-	gp_widget_set_value (widget, "This is some text.");
-	gp_widget_append (section, widget);
-
-	gp_widget_new (GP_WIDGET_DATE, "Date & Time", &widget);
-	val = time (NULL) - 100000;
-	gp_widget_set_value (widget, &val);
-	gp_widget_append (section, widget);
-#endif
-
-        return (GP_OK);
-}
-
-static int
-camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
-{
-        CameraWidget *widget;
-        char buf[256];
-        int val;
-
-        gp_widget_get_child_by_label (window, _("View hidden directories"),
-				      &widget);
-        if (gp_widget_changed (widget)) {
-                gp_widget_get_value (widget, &val);
-                sprintf (buf, "%i", val);
-                gp_setting_set ("directory", "hidden", buf);
-        }
-        return (GP_OK);
-}
-
 static int
 camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 {
@@ -562,13 +528,22 @@ make_dir_func (CameraFilesystem *fs, const char *folder, const char *name,
 	       void *data, GPContext *context)
 {
 	char path[2048];
+	Camera *camera = (Camera*)data;
 
-	strncpy (path, folder, sizeof (path));
-	if (strlen (folder) > 1)
-		strncat (path, "/", sizeof (path));
-	strncat (path, name, sizeof (path));
+	if (camera->port->type == GP_PORT_DISK) {
+		GPPortSettings settings;
 
-	return (GP_SYSTEM_MKDIR (path));
+		gp_port_get_settings (camera->port, &settings);
+		snprintf (path, sizeof(path), "%s/%s/%s",
+			settings.disk.mountpoint, 
+			folder,
+			name
+		);
+	} else {
+		/* old style access */
+		snprintf (path, sizeof (path), "%s/%s", folder, name);
+	}
+	return (gp_system_mkdir (path));
 }
 
 static int
@@ -576,13 +551,22 @@ remove_dir_func (CameraFilesystem *fs, const char *folder, const char *name,
 		 void *data, GPContext *context)
 {
 	char path[2048];
+	Camera *camera = (Camera*)data;
 
-	strncpy (path, folder, sizeof (path));
-	if (strlen (folder) > 1)
-		strncat (path, "/", sizeof (path));
-	strncat (path, name, sizeof (path));
+	if (camera->port->type == GP_PORT_DISK) {
+		GPPortSettings settings;
 
-	return (GP_SYSTEM_RMDIR (path));
+		gp_port_get_settings (camera->port, &settings);
+		snprintf (path, sizeof(path), "%s/%s/%s",
+			settings.disk.mountpoint, 
+			folder,
+			name
+		);
+	} else {
+		/* old style access */
+		snprintf (path, sizeof (path), "%s/%s", folder, name);
+	}
+	return (gp_system_rmdir (path));
 }
 
 static int
@@ -591,11 +575,21 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 {
 	char path[2048];
 	int result;
+	Camera *camera = (Camera*)data;
 
-	if (strlen (folder) > 1)
+	if (camera->port->type == GP_PORT_DISK) {
+		GPPortSettings settings;
+
+		gp_port_get_settings (camera->port, &settings);
+		snprintf (path, sizeof(path), "%s/%s/%s",
+			settings.disk.mountpoint, 
+			folder,
+			file
+		);
+	} else {
+		/* old style access */
 		snprintf (path, sizeof (path), "%s/%s", folder, file);
-	else
-		snprintf (path, sizeof (path), "/%s", file);
+	}
 	result = unlink (path);
 	if (result) {
 		gp_context_error (context, _("Could not delete file '%s' "
@@ -617,13 +611,23 @@ put_file_func (CameraFilesystem *fs, const char *folder,
 #ifdef DEBUG
 	unsigned int i, id;
 #endif
+	Camera *camera = (Camera*)data;
 
 	gp_file_get_name (file, &name);
 
-	strncpy (path, folder, sizeof (path));
-	if (strlen (folder) > 1)
-		strncat (path, "/", sizeof (path));
-	strncat (path, name, sizeof (path));
+	if (camera->port->type == GP_PORT_DISK) {
+		GPPortSettings settings;
+
+		gp_port_get_settings (camera->port, &settings);
+		snprintf (path, sizeof(path), "%s/%s/%s",
+			settings.disk.mountpoint, 
+			folder,
+			name
+		);
+	} else {
+		/* old style access */
+		snprintf (path, sizeof (path), "%s/%s", folder, name);
+	}
 
 	result = gp_file_save (file, path);
 	if (result < 0)
@@ -670,33 +674,23 @@ camera_capture (Camera *camera, CameraCaptureType type, CameraFilePath *path,
 }
 #endif
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.folder_list_func = folder_list_func,
+	.set_info_func = set_info_func,
+	.get_info_func = get_info_func,
+	.get_file_func = get_file_func,
+	.put_file_func = put_file_func,
+	.del_file_func = delete_file_func,
+	.make_dir_func = make_dir_func,
+	.remove_dir_func = remove_dir_func,
+};
+
 int
 camera_init (Camera *camera, GPContext *context)
 {
-        char buf[256];
-
         /* First, set up all the function pointers */
-        camera->functions->get_config           = camera_get_config;
-        camera->functions->set_config           = camera_set_config;
         camera->functions->manual               = camera_manual;
         camera->functions->about                = camera_about;
-#ifdef DEBUG
-	camera->functions->capture              = camera_capture;
-	camera->functions->capture_preview	= camera_capture_preview;
-#endif
-
-        if (gp_setting_get("directory", "hidden", buf) != GP_OK)
-                gp_setting_set("directory", "hidden", "1");
-
-        gp_filesystem_set_list_funcs (camera->fs, file_list_func,
-                                      folder_list_func, camera);
-	gp_filesystem_set_info_funcs (camera->fs, get_info_func,
-				      set_info_func, camera);
-	gp_filesystem_set_file_funcs (camera->fs, get_file_func,
-				      delete_file_func, camera);
-	gp_filesystem_set_folder_funcs (camera->fs, put_file_func, NULL,
-					make_dir_func, remove_dir_func, camera);
-
-        return (GP_OK);
+        return gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 }
-
