@@ -485,6 +485,14 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	return (GP_OK);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_file_func = get_file_func,
+	.del_file_func = del_file_func,
+	.put_file_func = put_file_func,
+	.get_info_func = get_info_func
+};
+
 int
 camera_init (Camera *camera, GPContext *context)
 {
@@ -517,18 +525,10 @@ camera_init (Camera *camera, GPContext *context)
 	CR (gp_port_set_settings (camera->port, settings));
 
 	/* Set up the filesystem. */
-	CR (gp_filesystem_set_list_funcs   (camera->fs, file_list_func, NULL,
-					    camera));
-	CR (gp_filesystem_set_file_funcs   (camera->fs, get_file_func,
-					    del_file_func, camera));
-	CR (gp_filesystem_set_info_funcs   (camera->fs, get_info_func, NULL,
-					    camera));
-	CR (gp_filesystem_set_folder_funcs (camera->fs, put_file_func, NULL,
-					    NULL, NULL, camera));
+	CR (gp_filesystem_set_funcs   (camera->fs, &fsfuncs, camera));
 
 	/* Initialize the connection */
 	CR (pre_func (camera, context));
-
 	/*
 	 * What commands does this camera support? The question is not
 	 * easy to answer, as "One issue the DS7 has is that the 
