@@ -170,6 +170,14 @@ camera_about (Camera *camera, CameraText *about, GPContext *context)
 	return (GP_OK);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_file_func = get_file_func,
+	.get_info_func = get_info_func,
+	.del_file_func = delete_file_func,
+	.delete_all_func = delete_all_func
+};
+
 int
 camera_init (Camera *camera, GPContext *context)
 {
@@ -197,7 +205,7 @@ camera_init (Camera *camera, GPContext *context)
 		default:
 			gp_context_error (context,
 					  _("Unsupported port type: %d. "
-					    "This driver only works with USB"
+					    "This driver only works with USB "
 					    "cameras.\n"), camera->port->type);
 			return (GP_ERROR);
 			break;
@@ -219,19 +227,8 @@ camera_init (Camera *camera, GPContext *context)
 
 		return (ret);
 	}
-
 	/* Set up the CameraFilesystem */
-	CHECK (gp_filesystem_set_list_funcs (camera->fs, file_list_func, 
-				             NULL, camera));
-	CHECK (gp_filesystem_set_file_funcs (camera->fs, get_file_func,
-					     delete_file_func, camera));
-	CHECK (gp_filesystem_set_info_funcs (camera->fs, get_info_func, 
-				             NULL, camera));
-	CHECK (gp_filesystem_set_folder_funcs (camera->fs, NULL, 
-			                       delete_all_func, NULL, NULL, 
-					       camera));
-
-	return (GP_OK);
+	return gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 }
 
 
