@@ -448,6 +448,13 @@ static int camera_about (Camera *camera, CameraText *about, GPContext *context)
 	return (GP_OK);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_file_func = get_file_func,
+	.get_info_func = get_info_func,
+	.folder_list_func = folder_list_func
+};
+
 int camera_init (Camera *camera, GPContext *context) 
 {
 	int ret;
@@ -462,10 +469,7 @@ int camera_init (Camera *camera, GPContext *context)
         camera->functions->about                = camera_about;
 
 	/* Set up the CameraFilesystem */
-	gp_filesystem_set_list_funcs (camera->fs, file_list_func,
-				      folder_list_func, camera);
-	gp_filesystem_set_file_funcs (camera->fs, get_file_func, NULL, camera);
-	gp_filesystem_set_info_funcs (camera->fs, get_info_func, NULL, camera);
+	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 
         /* initialize the camera */
 	ret = init (camera);
@@ -481,7 +485,6 @@ int camera_init (Camera *camera, GPContext *context)
 		camera->pl = NULL;
 		return (ret);
 	}
-
 	camera->pl->context = NULL;
 	return (GP_OK);
 }
