@@ -642,8 +642,8 @@ camera_get_config (Camera* camera, CameraWidget** window, GPContext *context)
         time_t          t;
         float value_float;
         const char *name;
-        GP_SYSTEM_DIR d;
-        GP_SYSTEM_DIRENT de;
+        gp_system_dir d;
+        gp_system_dirent de;
 	unsigned int id;
 
         /* Get the current settings. */
@@ -739,12 +739,12 @@ camera_get_config (Camera* camera, CameraWidget** window, GPContext *context)
         gp_widget_append (*window, section);
 
         /* Language */
-        d = GP_SYSTEM_OPENDIR (LOCALIZATION);
+        d = gp_system_opendir (LOCALIZATION);
         if (d) {
                 gp_widget_new (GP_WIDGET_MENU, _("Language"), &widget);
                 gp_widget_append (section, widget);
-                while ((de = GP_SYSTEM_READDIR (d))) {
-                        name = GP_SYSTEM_FILENAME (de);
+                while ((de = gp_system_readdir (d))) {
+                        name = gp_system_filename (de);
                         if (name && (*name != '.'))
                                 gp_widget_add_choice (widget, name);
                 }
@@ -1203,6 +1203,15 @@ camera_post_func (Camera *camera, GPContext *context)
 	return (GP_OK);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.get_info_func = get_info_func,
+	.set_info_func = set_info_func,
+	.file_list_func = file_list_func,
+	.get_file_func = get_file_func,
+	.del_file_func = delete_file_func,
+	.delete_all_func = delete_all_func,
+};
+
 int
 camera_init (Camera* camera, GPContext *context)
 {
@@ -1267,21 +1276,12 @@ camera_init (Camera* camera, GPContext *context)
         default:
                 return (GP_ERROR_UNKNOWN_PORT);
         }
-
         /* Set up the filesystem */
-	C(gp_filesystem_set_info_funcs (camera->fs, get_info_func,
-					set_info_func, camera));
-	C(gp_filesystem_set_list_funcs (camera->fs, file_list_func,
-					NULL, camera));
-	C(gp_filesystem_set_file_funcs (camera->fs, get_file_func,
-					delete_file_func, camera));
-	C(gp_filesystem_set_folder_funcs (camera->fs, NULL, delete_all_func,
-					NULL, NULL, camera));
+	C(gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera));
 
 	/* Ping the camera every minute to prevent shut-down. */
 	camera->pl->timeout = gp_camera_start_timeout (camera, PING_TIMEOUT,
 						       timeout_func);
-
         return (GP_OK);
 }
 
