@@ -379,7 +379,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			}
 			exifdat.header = d;
 			exifdat.data = d+12;
-			thumbnail = exif_get_thumbnail_and_size(&exifdat,
+			thumbnail = gpi_exif_get_thumbnail_and_size(&exifdat,
 				&long_len);
 			free(d);
 			d = thumbnail;
@@ -1254,6 +1254,15 @@ camera_abilities (CameraAbilitiesList *list)
 /*
  * Initialization of the camera
  */
+
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_info_func = get_info_func,
+	.get_file_func = get_file_func,
+	.put_file_func = put_file_func,
+	.del_file_func = delete_file_func,
+	.delete_all_func = delete_all_func,
+};
 int
 camera_init (Camera *camera, GPContext *context) 
 {
@@ -1272,12 +1281,7 @@ camera_init (Camera *camera, GPContext *context)
 
 
 	/* Now, tell the filesystem where to get lists, files and info */
-	gp_filesystem_set_list_funcs (camera->fs, file_list_func, NULL, camera);
-	gp_filesystem_set_info_funcs (camera->fs, get_info_func, NULL, camera);
-	gp_filesystem_set_file_funcs (camera->fs, get_file_func, 
-		delete_file_func, camera);
-	gp_filesystem_set_folder_funcs (camera->fs, put_file_func, 
-		delete_all_func, NULL, NULL, camera);
+	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 
 	/*
 	 * The port is already provided with camera->port (and
