@@ -640,6 +640,14 @@ static int camera_about (Camera *camera, CameraText *about, GPContext *context)
         return (GP_OK);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_info_func = get_info_func,
+	.get_file_func = get_file_func,
+	.put_file_func = put_file_func,
+	.del_file_func = delete_file_func,
+};
+
 int camera_init (Camera *camera, GPContext *context) 
 {
         GPPortSettings settings;
@@ -670,15 +678,7 @@ int camera_init (Camera *camera, GPContext *context)
         settings.serial.stopbits   = 1;
         CHECK (gp_port_set_settings (camera->port, settings));
 
-      	CHECK (gp_filesystem_set_list_funcs (camera->fs,
-		file_list_func, NULL, camera));
-	CHECK (gp_filesystem_set_info_funcs (camera->fs,
-		get_info_func, NULL, camera));
-	CHECK (gp_filesystem_set_file_funcs (camera->fs,
-		get_file_func, delete_file_func, camera));
-	CHECK (gp_filesystem_set_folder_funcs (camera->fs,
-		put_file_func, NULL, NULL, NULL, camera));
-
+      	CHECK (gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera));
 	/* Connect with the selected speed */
         return dsc2_connect(camera, selected_speed);
 }
