@@ -340,7 +340,7 @@ static int l859_readimageblock(Camera *camera, int block, char *buffer) {
 
 int camera_id (CameraText *id) {
 
-	GP_DEBUG ("Camera ID");
+	/* GP_DEBUG ("Camera ID"); */
 
 	strcpy(id->text, "panasonic-l859");
 
@@ -618,6 +618,13 @@ static int camera_about (Camera *camera, CameraText *about,
 	return (GP_OK);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_file_func = get_file_func,
+	.delete_all_func = delete_all_func,
+	.del_file_func = delete_file_func
+};
+
 int camera_init (Camera *camera, GPContext *context) {
 
         int ret;
@@ -644,11 +651,7 @@ int camera_init (Camera *camera, GPContext *context) {
         gp_port_set_settings(camera->port, settings);
 
 	/* Set up the filesystem */
-	gp_filesystem_set_list_funcs (camera->fs, file_list_func, NULL, camera);
-	gp_filesystem_set_file_funcs (camera->fs, get_file_func,
-				      delete_file_func, camera);
-	gp_filesystem_set_folder_funcs (camera->fs, NULL, delete_all_func,
-					NULL, NULL, camera);
+	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 
         ret = l859_connect (camera);
 	if (ret < 0) {
