@@ -291,6 +291,13 @@ delete_all_func (CameraFilesystem *fs, const char *folder, void *data,
 	return GP_OK;
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_info_func = get_info_func,
+	.get_file_func = get_file_func,
+	.del_file_func = delete_file_func,
+	.delete_all_func = delete_all_func,
+};
 
 int
 camera_init (Camera *camera, GPContext *context)
@@ -323,13 +330,5 @@ camera_init (Camera *camera, GPContext *context)
 			return GP_ERROR_NOT_SUPPORTED;
 	}
 	CHECK (pccam300_init (camera->port, context));
-
-	gp_filesystem_set_info_funcs (camera->fs, get_info_func, NULL, camera);
-	gp_filesystem_set_list_funcs (camera->fs, file_list_func, NULL, camera);
-	gp_filesystem_set_file_funcs (camera->fs, get_file_func,
-				      delete_file_func, camera);
-	CHECK (gp_filesystem_set_folder_funcs
-	       (camera->fs, NULL, delete_all_func, NULL, NULL, camera));
-
-	return GP_OK;
+	return gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 }
