@@ -160,6 +160,13 @@ static struct {
 			"trust%04i.ppm"
 		}
 	},
+	{"Clever CAM 360", 0x797, 0x8001, {
+			jd350e,
+			BAYER_TILE_BGGR,
+			&jd350e_postprocessing_and_flip,
+			"scope%04i.ppm"
+		}
+	},
 	/* http://www.meade.com/sportsoptics/catalog/captureview/index.html */
 	{"GrandTek ScopeCam", 0x797, 0x801c, {
 		 	jd350e,
@@ -1000,6 +1007,14 @@ camera_exit (Camera *camera, GPContext *context)
 	return (GP_OK);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_info_func = get_info_func,
+	.get_file_func = get_file_func,
+	.del_file_func = delete_file_func,
+	.delete_all_func = delete_all_func,
+};
+
 int
 camera_init (Camera *camera, GPContext *context)
 {
@@ -1034,16 +1049,8 @@ camera_init (Camera *camera, GPContext *context)
 	if( ! camera->pl ){
 		return (GP_ERROR_NOT_SUPPORTED);
 	}
-
 	/* Tell the filesystem where to get lists and info */
-	CHECK_RESULT (gp_filesystem_set_list_funcs (camera->fs, file_list_func,
-						    NULL, camera));
-	CHECK_RESULT (gp_filesystem_set_info_funcs (camera->fs, get_info_func,
-						    NULL, camera));
-	CHECK_RESULT (gp_filesystem_set_folder_funcs (camera->fs, NULL,
-					delete_all_func, NULL, NULL, camera));
-	CHECK_RESULT (gp_filesystem_set_file_funcs (camera->fs, get_file_func,
-						delete_file_func, camera));
+	CHECK_RESULT (gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera));
 
 	if (camera->port->type == GP_PORT_SERIAL) {
 		/* Open the port */
