@@ -1175,6 +1175,13 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *file,
 	return (GP_OK);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_info_func = get_info_func,
+	.get_file_func = get_file_func,
+	.del_file_func = del_file_func
+};
+
 int
 camera_init (Camera *camera, GPContext *context)
 {
@@ -1190,10 +1197,7 @@ camera_init (Camera *camera, GPContext *context)
 	camera->functions->set_config = camera_set_config;
 
 	/* Now, tell the filesystem where to get lists and info */
-	gp_filesystem_set_list_funcs (camera->fs, file_list_func, NULL, camera);
-	gp_filesystem_set_info_funcs (camera->fs, get_info_func, NULL, camera);
-	gp_filesystem_set_file_funcs (camera->fs, get_file_func,
-				      del_file_func, camera);
+	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 
 	/* Check if the camera is really there */
 	CR (gp_port_get_settings (camera->port, &settings));
