@@ -54,6 +54,22 @@ dtoh32ap (PTPParams *params, unsigned char *a)
 	return ((params->byteorder==PTP_DL_LE)?le32atoh(a):be32atoh(a));
 }
 
+static inline uint64_t
+dtoh64ap (PTPParams *params, unsigned char *a)
+{
+	uint64_t tmp = 0;
+	int i;
+
+	if (params->byteorder==PTP_DL_LE) {
+		for (i=0;i<8;i++)
+			tmp |= (((uint64_t)a[i]) << (8*i));
+	} else {
+		for (i=0;i<8;i++)
+			tmp |= (((uint64_t)a[i]) << (8*(7-i)));
+	}
+	return tmp;
+}
+
 #define htod8a(a,x)	*(uint8_t*)(a) = x
 #define htod16a(a,x)	htod16ap(params,a,x)
 #define htod32a(a,x)	htod32ap(params,a,x)
@@ -63,12 +79,13 @@ dtoh32ap (PTPParams *params, unsigned char *a)
 #define dtoh8a(x)	(*(uint8_t*)(x))
 #define dtoh16a(a)	dtoh16ap(params,a)
 #define dtoh32a(a)	dtoh32ap(params,a)
+#define dtoh64a(a)	dtoh64ap(params,a)
 #define dtoh16(x)	dtoh16p(params,x)
 #define dtoh32(x)	dtoh32p(params,x)
 
 
 static inline char*
-ptp_unpack_string(PTPParams *params, char* data, uint16_t offset, uint8_t *len)
+ptp_unpack_string(PTPParams *params, unsigned char* data, uint16_t offset, uint8_t *len)
 {
 	int i;
 	char *string=NULL;
@@ -87,7 +104,7 @@ ptp_unpack_string(PTPParams *params, char* data, uint16_t offset, uint8_t *len)
 }
 
 static inline void
-ptp_pack_string(PTPParams *params, char *string, char* data, uint16_t offset, uint8_t *len)
+ptp_pack_string(PTPParams *params, char *string, unsigned char* data, uint16_t offset, uint8_t *len)
 {
 	int i;
 	*len = (uint8_t)strlen(string);
@@ -100,7 +117,7 @@ ptp_pack_string(PTPParams *params, char *string, char* data, uint16_t offset, ui
 }
 
 static inline uint32_t
-ptp_unpack_uint32_t_array(PTPParams *params, char* data, uint16_t offset, uint32_t **array)
+ptp_unpack_uint32_t_array(PTPParams *params, unsigned char* data, uint16_t offset, uint32_t **array)
 {
 	uint32_t n, i=0;
 
@@ -114,7 +131,7 @@ ptp_unpack_uint32_t_array(PTPParams *params, char* data, uint16_t offset, uint32
 }
 
 static inline uint32_t
-ptp_unpack_uint16_t_array(PTPParams *params, char* data, uint16_t offset, uint16_t **array)
+ptp_unpack_uint16_t_array(PTPParams *params, unsigned char* data, uint16_t offset, uint16_t **array)
 {
 	uint32_t n, i=0;
 
@@ -137,7 +154,7 @@ ptp_unpack_uint16_t_array(PTPParams *params, char* data, uint16_t offset, uint16
 #define PTP_di_OperationsSupported	10
 
 static inline void
-ptp_unpack_DI (PTPParams *params, char* data, PTPDeviceInfo *di, unsigned int datalen)
+ptp_unpack_DI (PTPParams *params, unsigned char* data, PTPDeviceInfo *di, unsigned int datalen)
 {
 	uint8_t len;
 	unsigned int totallen;
@@ -196,7 +213,7 @@ ptp_unpack_DI (PTPParams *params, char* data, PTPDeviceInfo *di, unsigned int da
 #define PTP_oh				 0
 
 static inline void
-ptp_unpack_OH (PTPParams *params, char* data, PTPObjectHandles *oh, unsigned int len)
+ptp_unpack_OH (PTPParams *params, unsigned char* data, PTPObjectHandles *oh, unsigned int len)
 {
 	oh->n = ptp_unpack_uint32_t_array(params, data, PTP_oh, &oh->Handler);
 }
@@ -206,7 +223,7 @@ ptp_unpack_OH (PTPParams *params, char* data, PTPObjectHandles *oh, unsigned int
 #define PTP_sids			 0
 
 static inline void
-ptp_unpack_SIDs (PTPParams *params, char* data, PTPStorageIDs *sids, unsigned int len)
+ptp_unpack_SIDs (PTPParams *params, unsigned char* data, PTPStorageIDs *sids, unsigned int len)
 {
 	sids->n = ptp_unpack_uint32_t_array(params, data, PTP_sids,
 	&sids->Storage);
@@ -223,14 +240,15 @@ ptp_unpack_SIDs (PTPParams *params, char* data, PTPStorageIDs *sids, unsigned in
 #define PTP_si_StorageDescription	26
 
 static inline void
-ptp_unpack_SI (PTPParams *params, char* data, PTPStorageInfo *si, unsigned int len)
+ptp_unpack_SI (PTPParams *params, unsigned char* data, PTPStorageInfo *si, unsigned int len)
 {
 	uint8_t storagedescriptionlen;
 
 	si->StorageType=dtoh16a(&data[PTP_si_StorageType]);
 	si->FilesystemType=dtoh16a(&data[PTP_si_FilesystemType]);
 	si->AccessCapability=dtoh16a(&data[PTP_si_AccessCapability]);
-	/* XXX no dtoh64a !!! skiping next two */
+	si->MaxCapability=dtoh64a(&data[PTP_si_MaxCapability]);
+	si->FreeSpaceInBytes=dtoh64a(&data[PTP_si_FreeSpaceInBytes]);
 	si->FreeSpaceInImages=dtoh32a(&data[PTP_si_FreeSpaceInImages]);
 	si->StorageDescription=ptp_unpack_string(params, data,
 		PTP_si_StorageDescription, &storagedescriptionlen);
@@ -260,9 +278,9 @@ ptp_unpack_SI (PTPParams *params, char* data, PTPStorageInfo *si, unsigned int l
 #define PTP_oi_Filename			53
 
 static inline uint32_t
-ptp_pack_OI (PTPParams *params, PTPObjectInfo *oi, char** oidataptr)
+ptp_pack_OI (PTPParams *params, PTPObjectInfo *oi, unsigned char** oidataptr)
 {
-	char* oidata;
+	unsigned char* oidata;
 	uint8_t filenamelen;
 	uint8_t capturedatelen=0;
 	/* let's allocate some memory first; XXX i'm sure it's wrong */
@@ -322,7 +340,7 @@ ptp_pack_OI (PTPParams *params, PTPObjectInfo *oi, char** oidataptr)
 }
 
 static inline void
-ptp_unpack_OI (PTPParams *params, char* data, PTPObjectInfo *oi, unsigned int len)
+ptp_unpack_OI (PTPParams *params, unsigned char* data, PTPObjectInfo *oi, unsigned int len)
 {
 	uint8_t filenamelen;
 	uint8_t capturedatelen;
@@ -494,7 +512,7 @@ ptp_unpack_DPV (
 #define PTP_dpd_FactoryDefaultValue	5
 
 static inline int
-ptp_unpack_DPD (PTPParams *params, char* data, PTPDevicePropDesc *dpd, unsigned int dpdlen)
+ptp_unpack_DPD (PTPParams *params, unsigned char* data, PTPDevicePropDesc *dpd, unsigned int dpdlen)
 {
 	int offset=0, ret;
 
@@ -557,16 +575,87 @@ ptp_unpack_DPD (PTPParams *params, char* data, PTPDevicePropDesc *dpd, unsigned
 		}
 		}
 	}
+#undef N
 	return 1;
 outofmemory:
 	ptp_free_devicepropdesc(dpd);
 	return 0;
 }
 
-static inline uint32_t
-ptp_pack_DPV (PTPParams *params, PTPPropertyValue* value, char** dpvptr, uint16_t datatype)
+/* (MTP) Object Property pack/unpack */
+#define PTP_opd_ObjectPropertyCode	0
+#define PTP_opd_DataType		2
+#define PTP_opd_GetSet			4
+#define PTP_opd_FactoryDefaultValue	5
+
+static inline int
+ptp_unpack_OPD (PTPParams *params, unsigned char* data, PTPObjectPropDesc *opd, unsigned int opdlen)
 {
-	char* dpv=NULL;
+	int offset=0, ret;
+
+	memset (opd, 0, sizeof(*opd));
+	opd->ObjectPropertyCode=dtoh16a(&data[PTP_opd_ObjectPropertyCode]);
+	opd->DataType=dtoh16a(&data[PTP_opd_DataType]);
+	opd->GetSet=dtoh8a(&data[PTP_opd_GetSet]);
+
+	offset = PTP_opd_FactoryDefaultValue;
+	ret = ptp_unpack_DPV (params, data, &offset, opdlen, &opd->FactoryDefaultValue, opd->DataType);
+	if (!ret) goto outofmemory;
+
+	opd->GroupCode=dtoh32a(&data[offset]);
+	offset+=sizeof(uint32_t);
+
+	opd->FormFlag=dtoh8a(&data[offset]);
+	offset+=sizeof(uint8_t);
+
+	switch (opd->FormFlag) {
+	case PTP_OPFF_Range:
+		ret = ptp_unpack_DPV (params, data, &offset, opdlen, &opd->FORM.Range.MinimumValue, opd->DataType);
+		if (!ret) goto outofmemory;
+		ret = ptp_unpack_DPV (params, data, &offset, opdlen, &opd->FORM.Range.MaximumValue, opd->DataType);
+		if (!ret) goto outofmemory;
+		ret = ptp_unpack_DPV (params, data, &offset, opdlen, &opd->FORM.Range.StepSize, opd->DataType);
+		if (!ret) goto outofmemory;
+		break;
+	case PTP_OPFF_Enumeration: {
+		int i;
+#define N	opd->FORM.Enum.NumberOfValues
+		N = dtoh16a(&data[offset]);
+		offset+=sizeof(uint16_t);
+		opd->FORM.Enum.SupportedValue = malloc(N*sizeof(opd->FORM.Enum.SupportedValue[0]));
+		if (!opd->FORM.Enum.SupportedValue)
+			goto outofmemory;
+
+		memset (opd->FORM.Enum.SupportedValue,0 , N*sizeof(opd->FORM.Enum.SupportedValue[0]));
+		for (i=0;i<N;i++) {
+			ret = ptp_unpack_DPV (params, data, &offset, opdlen, &opd->FORM.Enum.SupportedValue[i], opd->DataType);
+
+			/* Slightly different handling here. The HP PhotoSmart 120
+			 * specifies an enumeration with N in wrong endian
+			 * 00 01 instead of 01 00, so we count the enum just until the
+			 * the end of the packet.
+			 */
+			if (!ret) {
+				if (!i)
+					goto outofmemory;
+				opd->FORM.Enum.NumberOfValues = i;
+				break;
+			}
+		}
+#undef N
+		}
+	}
+	return 1;
+outofmemory:
+	ptp_free_objectpropdesc(opd);
+	return 0;
+}
+
+
+static inline uint32_t
+ptp_pack_DPV (PTPParams *params, PTPPropertyValue* value, unsigned char** dpvptr, uint16_t datatype)
+{
+	unsigned char* dpv=NULL;
 	uint32_t size=0;
 	int	i;
 
@@ -672,7 +761,7 @@ ptp_pack_DPV (PTPParams *params, PTPPropertyValue* value, char** dpvptr, uint16_
 #define PTP_ec_Param3		20
 
 static inline void
-ptp_unpack_EC (PTPParams *params, char* data, PTPUSBEventContainer *ec, unsigned int len)
+ptp_unpack_EC (PTPParams *params, unsigned char* data, PTPUSBEventContainer *ec, unsigned int len)
 {
 	if (data==NULL)
 		return;
@@ -707,7 +796,7 @@ ptp_unpack_EC (PTPParams *params, char* data, PTPUSBEventContainer *ec, unsigned
 #define PTP_cfe_Filename		15
 
 static inline void
-ptp_unpack_Canon_FE (PTPParams *params, char* data, PTPCANONFolderEntry *fe)
+ptp_unpack_Canon_FE (PTPParams *params, unsigned char* data, PTPCANONFolderEntry *fe)
 {
 	int i;
 	if (data==NULL)
@@ -715,10 +804,79 @@ ptp_unpack_Canon_FE (PTPParams *params, char* data, PTPCANONFolderEntry *fe)
 	fe->ObjectHandle=dtoh32a(&data[PTP_cfe_ObjectHandle]);
 	fe->ObjectFormatCode=dtoh16a(&data[PTP_cfe_ObjectFormatCode]);
 	fe->Flags=dtoh8a(&data[PTP_cfe_Flags]);
-	fe->ObjectSize=dtoh32a(&data[PTP_cfe_ObjectSize]);
+	fe->ObjectSize=dtoh32a((unsigned char*)&data[PTP_cfe_ObjectSize]);
 	fe->Time=(time_t)dtoh32a(&data[PTP_cfe_Time]);
 	for (i=0; i<PTP_CANON_FilenameBufferLen; i++)
-	fe->Filename[i]=(char)dtoh8a(&data[PTP_cfe_Filename+i]);
+		fe->Filename[i]=(char)dtoh8a(&data[PTP_cfe_Filename+i]);
+}
+
+/*
+    PTP USB Event container unpack for Nikon events.
+*/
+#define PTP_nikon_ec_Length		0
+#define PTP_nikon_ec_Code		2
+#define PTP_nikon_ec_Param1		4
+#define PTP_nikon_ec_Size		6
+static inline void
+ptp_unpack_Nikon_EC (PTPParams *params, unsigned char* data, unsigned int len, PTPUSBEventContainer **ec, int *cnt)
+{
+	int i;
+
+	*ec = NULL;
+	if (data == NULL)
+		return;
+	if (len < PTP_nikon_ec_Code)
+		return;
+	*cnt = dtoh16a(&data[PTP_nikon_ec_Length]);
+	if (*cnt > (len-PTP_nikon_ec_Code)/PTP_nikon_ec_Size) /* broken cnt? */
+		return;
+	*ec = malloc(sizeof(PTPUSBEventContainer)*(*cnt));
+	
+	for (i=0;i<*cnt;i++) {
+		memset(&(*ec)[i],0,sizeof(PTPUSBEventContainer));
+		(*ec)[i].code	= dtoh16a(&data[PTP_nikon_ec_Code+PTP_nikon_ec_Size*i]);
+		(*ec)[i].param1	= dtoh32a(&data[PTP_nikon_ec_Param1+PTP_nikon_ec_Size*i]);
+	}
 }
 
 
+static inline uint32_t
+ptp_pack_EK_text(PTPParams *params, PTPEKTextParams *text, unsigned char **data) {
+	int i, len = 0;
+	uint8_t	retlen;
+	unsigned char *curdata;
+
+	len =	2*(strlen(text->title)+1)+1+
+		2*(strlen(text->line[0])+1)+1+
+		2*(strlen(text->line[1])+1)+1+
+		2*(strlen(text->line[2])+1)+1+
+		2*(strlen(text->line[3])+1)+1+
+		2*(strlen(text->line[4])+1)+1+
+		4*2+2*4+2+4+2+5*4*2;
+	*data = malloc(len);
+	if (!*data) return 0;
+
+	curdata = *data;
+	htod16a(curdata,100);curdata+=2;
+	htod16a(curdata,1);curdata+=2;
+	htod16a(curdata,0);curdata+=2;
+	htod16a(curdata,1000);curdata+=2;
+
+	htod32a(curdata,0);curdata+=4;
+	htod32a(curdata,0);curdata+=4;
+
+	htod16a(curdata,6);curdata+=2;
+	htod32a(curdata,0);curdata+=4;
+
+	ptp_pack_string(params, text->title, curdata, 0, &retlen); curdata+=2*retlen+1;htod16a(curdata,0);curdata+=2;
+	htod16a(curdata,0x10);curdata+=2;
+	
+	for (i=0;i<5;i++) {
+		ptp_pack_string(params, text->line[i], curdata, 0, &retlen); curdata+=2*retlen+1;htod16a(curdata,0);curdata+=2;
+		htod16a(curdata,0x10);curdata+=2;
+		htod16a(curdata,0x01);curdata+=2;
+		htod16a(curdata,0x02);curdata+=2;
+		htod16a(curdata,0x06);curdata+=2;
+	}
+	return len;
+}
