@@ -170,7 +170,7 @@ g3_channel_write(GPPort *port, int channel, char *buffer, int len)
 		memcpy(xbuf+8, buffer + curlen, sendlen);
 		curlen += sendlen;
 		xbuf[8+sendlen] = 0x03;
-		ret = gp_port_write( port, xbuf, nlen);
+		ret = gp_port_write( port, (char*)xbuf, nlen);
 		free(xbuf);
 		if (ret < GP_OK) break;
 		len -= sendlen;
@@ -218,7 +218,7 @@ camera_abilities (CameraAbilitiesList *list)
 
 	memset(&a, 0, sizeof(a));
 	strcpy(a.model, "Ricoh:Caplio G3");
-	a.status	= GP_DRIVER_STATUS_TESTING;
+	a.status	= GP_DRIVER_STATUS_PRODUCTION;
 	a.port		= GP_PORT_USB;
 	a.usb_vendor	= 0x5ca;
 	a.usb_product	= 0x2204;
@@ -240,6 +240,11 @@ camera_abilities (CameraAbilitiesList *list)
 	a.usb_product	= 0x2203;
 	gp_abilities_list_append(list, a);
 
+	strcpy(a.model, "Medion:MD 6126");
+	a.usb_vendor    = 0x5ca;
+	a.usb_product   = 0x2205;
+	gp_abilities_list_append(list, a);
+
 	strcpy(a.model, "Ricoh:Caplio G4");
 	a.usb_vendor	= 0x5ca;
 	a.usb_product	= 0x2208;
@@ -250,6 +255,11 @@ camera_abilities (CameraAbilitiesList *list)
 	a.usb_product	= 0x220b;
 	gp_abilities_list_append(list, a);
 
+	strcpy(a.model, "Ricoh:Caplio GX");
+	a.usb_vendor    = 0x5ca;
+	a.usb_product   = 0x220c;
+	gp_abilities_list_append(list, a);
+
 	strcpy(a.model, "Ricoh:Caplio R1");
 	a.usb_vendor	= 0x5ca;
 	a.usb_product	= 0x220d;
@@ -266,14 +276,29 @@ camera_abilities (CameraAbilitiesList *list)
 	a.usb_product	= 0x220e;
 	gp_abilities_list_append(list, a);
 
+	strcpy(a.model, "Rollei:dr5");
+	a.usb_vendor    = 0x5ca;
+	a.usb_product   = 0x220f;
+	gp_abilities_list_append(list, a);
+
 	strcpy(a.model, "Ricoh:Caplio R1v");
 	a.usb_vendor	= 0x5ca;
 	a.usb_product	= 0x2212;
 	gp_abilities_list_append(list, a);
 
-	strcpy(a.model, "Medion:MD 6126");
-	a.usb_vendor    = 0x5ca;
-	a.usb_product   = 0x2205;
+	strcpy(a.model, "Ricoh:Caplio R2");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2213;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Caplio GX 8");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2214;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Caplio R3");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2216;
 	gp_abilities_list_append(list, a);
 
 	return (GP_OK);
@@ -653,16 +678,6 @@ out:
 	return (GP_OK);
 
 }
-static int
-set_info_func (CameraFilesystem *fs, const char *folder, const char *file,
-	       CameraFileInfo info, void *data, GPContext *context)
-{
-	/*Camera *camera = data;*/
-
-	/* Set the file info here from <info> */
-
-	return (GP_OK);
-}
 
 
 static int
@@ -682,6 +697,10 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		 */
 		ret = g3_ftp_command_and_reply(camera->port, "-NLST /", &buf);
 		if (ret < GP_OK) goto out;
+		if (buf[0] == '4') {
+			/* seen with R3: 450 Open Error... likely OK. */
+			goto out;
+		}
 		if (buf[0] != '1') {
 			ret = GP_ERROR_IO;
 			goto out;
@@ -725,7 +744,10 @@ folder_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 			}
 		}
 	} else {
-		ret = GP_ERROR_IO;
+		if (buf[0] == '4')
+			ret = GP_OK;
+		else	
+			ret = GP_ERROR_IO;
 	}
 out:
 	if (buf) free(buf);
@@ -822,7 +844,10 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 			}
 		}
 	} else {
-		ret = GP_ERROR_IO;
+		if (buf[0] == '4') /* 450 Open Error ... like dir not there */
+			ret = GP_OK;
+		else		
+			ret = GP_ERROR_IO;
 	}
 out:
 	if (buf) free(buf);
@@ -831,6 +856,16 @@ out:
 	return (GP_OK);
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_file_func = get_file_func,
+	.del_file_func = delete_file_func,
+	.get_info_func = get_info_func,
+	.folder_list_func = folder_list_func,
+	.make_dir_func = mkdir_func,
+	.remove_dir_func = rmdir_func,
+};
+
 int
 camera_init (Camera *camera, GPContext *context) 
 {
@@ -842,15 +877,7 @@ camera_init (Camera *camera, GPContext *context)
         camera->functions->about                = camera_about;
 
 	/* Now, tell the filesystem where to get lists, files and info */
-	gp_filesystem_set_list_funcs (camera->fs, file_list_func,
-				      folder_list_func, camera);
-	gp_filesystem_set_info_funcs (camera->fs, get_info_func, set_info_func,
-				      camera);
-	gp_filesystem_set_file_funcs (camera->fs, get_file_func,
-				      delete_file_func, camera);
-	gp_filesystem_set_folder_funcs (camera->fs, NULL, NULL,
-					mkdir_func, rmdir_func, camera);
-
+	gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 
         gp_port_get_settings(camera->port, &settings);
 	settings.usb.inep = 0x81;
