@@ -86,7 +86,7 @@ static struct {
 	{"Epson", "PhotoPC 600", SIERRA_MODEL_EPSON,	0, 0, 0 },
 	{"Epson", "PhotoPC 650", SIERRA_MODEL_EPSON,	0, 0,
 					SIERRA_NO_51 | SIERRA_LOW_SPEED },
-	{"Epson", "PhotoPC 700", SIERRA_MODEL_EPSON,	0, 0, 0 },
+	{"Epson", "PhotoPC 700", SIERRA_MODEL_EPSON,	0, 0, SIERRA_NO_51 },
 	{"Epson", "PhotoPC 800", SIERRA_MODEL_EPSON,	0, 0, 0 },
 	{"Epson", "PhotoPC 850z", SIERRA_MODEL_EPSON,     0x4b8, 0x402, 0},
 	{"Epson", "PhotoPC 3000z", SIERRA_MODEL_CAM_DESC,	0x4b8, 0x403, 0, &ep3000z_cam_desc},
@@ -105,9 +105,11 @@ static struct {
 	{"Nikon", "CoolPix 995",   SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0104,
 						SIERRA_WRAP_USB_OLYMPUS,
 							&cp995_cam_desc},
+#if 0
 	{"Nikon", "CoolPix 2500 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0108,
 						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
-							&cp880_cam_desc},
+							&cp2500_cam_desc},
+#endif
 	{"Nikon", "CoolPix 4300 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x010e,
 						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
 							&cp880_cam_desc},
@@ -140,6 +142,8 @@ static struct {
 					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "C-750UZ",	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
 					SIERRA_WRAP_USB_OLYMPUS, &oly750uz_cam_desc},
+	{"Olympus", "C-770UZ",	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
+					SIERRA_WRAP_USB_OLYMPUS, &oly750uz_cam_desc},
 	{"Olympus", "C-800", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-800L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-820", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
@@ -150,6 +154,11 @@ static struct {
 	{"Olympus", "C-860L",	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-900 Zoom", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-900L Zoom", SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+
+	/* http://sourceforge.net/tracker/index.php?func=detail&aid=1082569&group_id=8874&atid=358874 
+	 * strangely only works with low speed
+	 */
+	{"Olympus", "C-990 Zoom", 	SIERRA_MODEL_OLYMPUS,	0, 0, SIERRA_LOW_SPEED },
 	{"Olympus", "C-1000L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-1400L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-1400XL", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
@@ -178,6 +187,13 @@ static struct {
 					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "C-5050Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
 					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
+	{"Olympus", "C-370Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x109,
+				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
+	{"Olympus", "X-450", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x109,
+				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
+	{"Olympus", "D-535Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x109,
+				SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
+
 	{"Panasonic", "Coolshot NV-DCF5E", SIERRA_MODEL_DEFAULT, 0, 0, 0 },
 
 	{"Pentax", "Optio 450", SIERRA_MODEL_DEFAULT, 0x0a17,0x0007, SIERRA_WRAP_USB_PENTAX },
@@ -355,6 +371,12 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		info->audio.fields |= GP_FILE_INFO_TYPE;
 	}
 
+#if 0 /* need to add GMT -> local conversion first */
+	if (i.date) {
+		info->file.fields |= GP_FILE_INFO_MTIME;
+		info->file.mtime = i.date; /* FIXME: convert from GMT to localtime */
+	}
+#endif
 	/* Type of image and preview? */
 	if (strstr (filename, ".MOV") != NULL) {
 		strcpy (info->file.type, GP_MIME_QUICKTIME);
@@ -1970,7 +1992,7 @@ camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 		    "    camera control menu appears. Set it to ON.\n"
 		    "(3) If you switch the 'LCD mode' to 'Monitor' or\n"
 		    "    'Normal', don't forget to switch it back to 'Off'\n"
-		    "    before disconnectig. Otherwise you cannot use\n"
+		    "    before disconnecting. Otherwise you cannot use\n"
 		    "    the camera's buttons. If you end up with this\n"
 		    "    state, you should reconnect the camera to the\n"
 		    "    PC and switch LCD to 'Off'."));
@@ -2040,6 +2062,17 @@ int get_jpeg_data(const char *data, int data_size, char **jpeg_data, int *jpeg_s
 	return ret_status;
 }
 
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.folder_list_func = folder_list_func,
+	.get_info_func = get_info_func,
+	.set_info_func = set_info_func,
+	.get_file_func = get_file_func,
+	.put_file_func = put_file_func,
+	.del_file_func = delete_file_func,
+	.delete_all_func = delete_all_func,
+};
+
 int
 camera_init (Camera *camera, GPContext *context) 
 {
@@ -2196,14 +2229,7 @@ camera_init (Camera *camera, GPContext *context)
 
         /* We start off not knowing where we are */
         strcpy (camera->pl->folder, "");
-        CHECK_STOP_FREE (camera, gp_filesystem_set_list_funcs (camera->fs,
-				file_list_func, folder_list_func, camera));
-        CHECK_STOP_FREE (camera, gp_filesystem_set_info_funcs (camera->fs,
-                                get_info_func, set_info_func, camera));
-	CHECK_STOP_FREE (camera, gp_filesystem_set_file_funcs (camera->fs,
-				get_file_func, delete_file_func, camera));
-	CHECK_STOP_FREE (camera, gp_filesystem_set_folder_funcs (camera->fs,
-				put_file_func, delete_all_func, NULL, NULL, camera));
+        CHECK_STOP_FREE (camera, gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera));
 
         CHECK (camera_stop (camera, context));
 
