@@ -1,5 +1,5 @@
 /* Sony DSC-F55 & MSAC-SR1 - gPhoto2 camera library
- * Copyright  2001, 2002 Raymond Penners <raymond@dotsphinx.com>
+ * Copyright  2001, 2002, 2004 Raymond Penners <raymond@dotsphinx.com>
  * Copyright  2000 Mark Davies <mdavies@dial.pipex.com>
  *
  * This library is free software; you can redistribute it and/or
@@ -43,12 +43,6 @@
 #define SONY_START_CHAR		0xc0
 #define SONY_END_CHAR		0xc1
 
-enum
-{
-	SONY_FILE_EXIF=0,
-	SONY_FILE_THUMBNAIL,
-	SONY_FILE_IMAGE
-};
 
 static unsigned char START_PACKET = 192;
 static unsigned char END_PACKET = 193;
@@ -131,7 +125,8 @@ static const int baud_rate = 9600;
 /**
  * Returns transfer rate ID
  */
-static int sony_baud_to_id(long baud)
+static int
+sony_baud_to_id(long baud)
 {
 	int r;
 
@@ -160,7 +155,8 @@ static int sony_baud_to_id(long baud)
 /**
  * Reads a byte
  */
-static int sony_read_byte(Camera * camera, unsigned char *b)
+static int
+sony_read_byte(Camera * camera, unsigned char *b)
 {
 	int n = gp_port_read(camera->port, b, 1);
 	if (n != 1)
@@ -174,7 +170,8 @@ static int sony_read_byte(Camera * camera, unsigned char *b)
 /**
  * Returns the checksum for a packet
  */
-static unsigned char sony_packet_checksum(Packet * p)
+static unsigned char
+sony_packet_checksum(Packet * p)
 {
 	unsigned short int o = 0;
 	unsigned long int sum = 0;
@@ -190,7 +187,8 @@ static unsigned char sony_packet_checksum(Packet * p)
 /**
  * Returns TRUE iff the packet is valid
  */
-static int sony_packet_validate(Camera * camera, Packet * p)
+static int
+sony_packet_validate(Camera * camera, Packet * p)
 {
 	unsigned char c = sony_packet_checksum(p);
 
@@ -301,7 +299,8 @@ static int sony_packet_read(Camera * camera, Packet * pack)
 /**
  * Sends a packet
  */
-static int sony_packet_write(Camera * camera, Packet * p)
+static int
+sony_packet_write(Camera * camera, Packet * p)
 {
 	unsigned short int count;
 	int rc;
@@ -309,6 +308,15 @@ static int sony_packet_write(Camera * camera, Packet * p)
 	GP_DEBUG(
 			"sony_packet_write()");
 
+        /**
+	 * This usleep is necessary when using (relatively) fast CPUs,
+	 * as P4 2Ghz. Without it gphoto2 stops with a segmentation fault
+	 * and the --debug message sony_packet_validate: invalid sequence
+	 * Failed to read packet during transfer. The delay is somewhat
+	 * arbitrary. (alberto-g@(remove_me)inventati.org)
+	 **/
+	usleep(10000);
+
 	rc = gp_port_write(camera->port, &START_PACKET, 1);
 
 	p->buffer[p->length] = p->checksum;
@@ -375,7 +383,8 @@ sony_converse(Camera * camera, Packet * out, unsigned char *str, int len)
 					break;
 
 				case SONY_INVALID_SEQUENCE:
-					if (camera->pl->msac_sr1) {
+					if (camera->pl->model
+					    != SONY_MODEL_DSC_F55) {
 						invalid_sequence = 1;
 						sony_packet_make(camera,
 								 &ps, str,
@@ -441,7 +450,8 @@ sony_converse(Camera * camera, Packet * out, unsigned char *str, int len)
 /**
  * Sets baud rate
  */
-static int sony_baud_port_set(Camera * camera, long baud)
+static int
+sony_baud_port_set(Camera * camera, long baud)
 {
 	gp_port_settings settings;
 
@@ -457,21 +467,32 @@ static int sony_baud_port_set(Camera * camera, long baud)
 /**
  * Sets baud rate
  */
-static int sony_baud_set(Camera * camera, long baud)
+static int
+sony_baud_set(Camera * camera, long baud)
 {
 	Packet dp;
 	int rc;
 
 	GP_DEBUG( "sony_baud_set(%ld)",
 			baud);
-	/* FIXME */
-	SetTransferRate[3] = sony_baud_to_id(baud);
 
-	rc = sony_converse(camera, &dp, SetTransferRate, 4);
-	if (rc == GP_OK) {
-		sony_baud_port_set(camera, baud);
-		rc = sony_converse(camera, &dp, EmptyPacket, 1);
-		usleep(100000);	/* 50000 was good too, jw */
+	if (camera->pl->current_baud_rate != baud) {
+		/* FIXME */
+		SetTransferRate[3] = sony_baud_to_id(baud);
+	
+		rc = sony_converse(camera, &dp, SetTransferRate, 4);
+		if (rc == GP_OK) {
+			sony_baud_port_set(camera, baud);
+			rc = sony_converse(camera, &dp, EmptyPacket, 1);
+			if (rc == GP_OK) {
+				camera->pl->current_baud_rate = baud;
+			}
+			usleep(100000);	/* 50000 was good too, jw */
+
+		}
+	}
+	else {
+		rc = GP_OK;
 	}
 	return rc;
 }
@@ -479,7 +500,8 @@ static int sony_baud_set(Camera * camera, long baud)
 /**
  * Port initialisation
  */
-static int sony_init_port (Camera *camera)
+static int
+sony_init_port (Camera *camera)
 {
 	gp_port_settings settings;
 	int rc;
@@ -505,7 +527,8 @@ static int sony_init_port (Camera *camera)
 /**
  * Establish first contact (remember the prime directive? :)
  */
-static int sony_init_first_contact (Camera *camera)
+static int
+sony_init_first_contact (Camera *camera)
 {
 	int count = 0;
 	Packet dp;
@@ -527,12 +550,25 @@ static int sony_init_first_contact (Camera *camera)
 	return rc;
 }
 
+/**
+ * Device supports MPEG?
+ */
+static int
+sony_is_mpeg_supported (Camera * camera)
+{
+	return camera->pl->model == SONY_MODEL_DSC_F55;
+}
+
 /**
  * Initialises camera
  */
-int sony_init (Camera * camera, int msac)
+int
+sony_init (Camera * camera, SonyModel model)
 {
 	int rc;
+	camera->pl->model = model;
+	camera->pl->current_baud_rate = -1;
+	camera->pl->current_mpeg_mode = -1;
 
 	rc = sony_init_port (camera);
 	if (rc == GP_OK)
@@ -544,7 +580,8 @@ int sony_init (Camera * camera, int msac)
 /**
  * Reset the camera sequence count and baud rate.
  */
-int sony_exit(Camera * camera)
+int
+sony_exit(Camera * camera)
 {
 	Packet dp;
 	int rc = GP_ERROR;
@@ -557,49 +594,105 @@ int sony_exit(Camera * camera)
 	return rc;
 }
 
+
+static int
+sony_set_file_mode(Camera * camera, SonyFileType file_type)
+{
+	int rc = GP_OK;
+	Packet dp;
+	if (file_type == SONY_FILE_MPEG) {
+		if (camera->pl->current_mpeg_mode != 1) {
+			rc = sony_converse(camera, &dp, MpegImage, 21);
+			if (rc == GP_OK) {
+				camera->pl->current_mpeg_mode = 1;
+			}
+		}
+	}
+	else {
+		if (camera->pl->current_mpeg_mode != 0) {
+			rc = sony_converse(camera, &dp, StillImage, 19);
+			if (rc == GP_OK) {
+				camera->pl->current_mpeg_mode = 0;
+			}
+		}
+	}
+	return rc;
+}
+
+
 /**
  * Return count of images taken.
  */
-static int
-sony_item_count(Camera * camera, unsigned char *from, int from_len)
+int
+sony_file_count(Camera * camera, SonyFileType file_type, int *count)
 {
 	Packet dp;
 	int rc;
 
-	GP_DEBUG( "sony_item_count()");
-	rc = sony_converse(camera, &dp, SetTransferRate, 4);
-	if (rc == GP_OK) {
-		rc = sony_converse(camera, &dp, from, from_len);
+	GP_DEBUG( "sony_file_count()");
+	if (file_type == SONY_FILE_MPEG
+	    && (! sony_is_mpeg_supported(camera))) {
+		    rc = GP_OK;
+		    *count = 0;
+	}
+	else {
+		*count = -1;
+		rc = sony_converse(camera, &dp, SetTransferRate, 4);
 		if (rc == GP_OK) {
-			rc = sony_converse(camera, &dp, SendImageCount, 3);
+			int rc = sony_set_file_mode(camera, file_type);
 			if (rc == GP_OK) {
-				int nr = dp.buffer[5] | (dp.buffer[4]<<8);
-				GP_DEBUG ("count = %d", nr);
-				return nr;
+				rc = sony_converse(camera, &dp, SendImageCount, 3);
+				if (rc == GP_OK) {
+					int nr = dp.buffer[5] | (dp.buffer[4]<<8);
+					GP_DEBUG ("count = %d", nr);
+					*count = nr;
+				}
 			}
 		}
 	}
-	return GP_ERROR;
+	return rc;
+}
+
+
+
+/**
+ * Fetches file name.
+ */
+int
+sony_file_name_get(Camera *camera, int imageid, SonyFileType mpeg, char buf[13])
+{
+       Packet dp;
+       int rc;
+
+       GP_DEBUG( "sony_file_name_get()");
+       rc = sony_set_file_mode(camera, mpeg);
+       if (rc == GP_OK) {
+	       sony_baud_set(camera, baud_rate);
+	       /* FIXME: Not nice, changing global data like this. */
+	       SelectImage[3] = (imageid >> 8);
+	       SelectImage[4] = imageid & 0xff;
+	       rc = sony_converse(camera, &dp, SelectImage, 7);
+	       if (rc == GP_OK) {
+		       memcpy(buf, &dp.buffer[5], 8);
+		       buf[8] = '.';
+		       memcpy(buf+9, &dp.buffer[5+8], 3);
+		       buf[12] = 0;
+	       }
+       }
+       return rc;
 }
 
 
 /**
- * Returns number of still images
+ * Is it an MPEG file?
  */
-int sony_image_count(Camera * camera)
+int
+sony_is_mpeg_file_name(const char * file_name)
 {
-	return sony_item_count(camera, StillImage, sizeof(StillImage));
+	return strncmp(file_name,"MOV",3)==0;
 }
 
 
-/**
- * Returns number of still images
- */
-int sony_mpeg_count(Camera * camera)
-{
-	return sony_item_count(camera, MpegImage, sizeof(MpegImage));
-}
-
 
 /**
  * Fetches an image.
@@ -611,9 +704,15 @@ sony_file_get(Camera * camera, int imageid, int file_type,
 	int sc;			/* count of bytes to skip at start of packet */
 	Packet dp;
 	int rc;
-	char buffer[128];
+	char buffer[128];                                                 
 
 	GP_DEBUG( "sony_file_get()");
+
+	rc = sony_set_file_mode(camera, file_type);
+	if (rc != GP_OK) {
+		return rc;
+	}
+	
 	if (gp_context_cancel(context) == GP_CONTEXT_FEEDBACK_CANCEL) {
 		return GP_ERROR_CANCEL;
 	}
@@ -621,24 +720,25 @@ sony_file_get(Camera * camera, int imageid, int file_type,
 	rc = gp_file_clean(file);
 	if (rc == GP_OK) {
 		gp_file_set_mime_type (file, GP_MIME_JPEG);
-		/* FIXME: There appears to be a file name at dp.buffer+5 */
-		/* after a SelectImage. We could use that for the real */
-		/* file name instead of our own mockup ("dsc%d.jpg"). */
+		/* The file name we specify here is not important,
+		 * since the calling function, gp_filesystem_get_file_impl,
+		 * overwrites it with the correct one.
+		 */
 		sprintf(buffer, SONY_FILE_NAME_FMT, imageid);
 		gp_file_set_name (file, buffer);
 
 		sony_baud_set(camera, baud_rate);
 
-		rc = sony_converse(camera, &dp, StillImage, 19);
+		rc = sony_set_file_mode(camera, file_type);
 		if (rc == GP_OK) {
 			if (file_type == SONY_FILE_THUMBNAIL) {
 				sc = 0x247;
-
 				SelectImage[3] = (imageid >> 8);
 				SelectImage[4] = imageid & 0xff;
 				sony_converse(camera, &dp, SelectImage, 7);
 
-				if (camera->pl->msac_sr1) {
+				if (camera->pl->model
+				    != SONY_MODEL_DSC_F55) {
 					gp_file_append(file,
 						       "\xff\xd8\xff", 3);
 				}
@@ -716,9 +816,10 @@ sony_file_get(Camera * camera, int imageid, int file_type,
 }
 
 /**
- * Fetches an image.
+ * Fetches a thumbnail image.
  */
-int sony_thumbnail_get(Camera * camera, int imageid, CameraFile * file, GPContext *context)
+int
+sony_thumbnail_get(Camera * camera, int imageid, CameraFile * file, GPContext *context)
 {
 	return sony_file_get(camera, imageid, SONY_FILE_THUMBNAIL, file, context);
 }
@@ -726,7 +827,8 @@ int sony_thumbnail_get(Camera * camera, int imageid, CameraFile * file, GPContex
 /**
  * Fetches an image.
  */
-int sony_image_get(Camera * camera, int imageid, CameraFile * file, GPContext *context)
+int
+sony_image_get(Camera * camera, int imageid, CameraFile * file, GPContext *context)
 {
 	return sony_file_get(camera, imageid, SONY_FILE_IMAGE, file, context);
 }
@@ -734,21 +836,38 @@ int sony_image_get(Camera * camera, int imageid, CameraFile * file, GPContext *c
 /**
  * Fetches EXIF information.
  */
-int sony_exif_get(Camera * camera, int imageid, CameraFile * file, GPContext *context)
+int
+sony_exif_get(Camera * camera, int imageid, CameraFile * file, GPContext *context)
 {
 	return sony_file_get(camera, imageid, SONY_FILE_EXIF, file, context);
 }
 
+/**
+ * Fetches an Mpeg.
+ */
+int
+sony_mpeg_get(Camera * camera, int imageid, CameraFile * file, GPContext *context)
+{
+	return sony_file_get(camera, imageid, SONY_FILE_MPEG, file, context);
+}
+
 
 /**
  * Fetches image details.
  */
-int sony_image_info(Camera * camera, int imageid, CameraFileInfo * info, GPContext *context)
+int
+sony_image_info(Camera * camera, int imageid, SonyFileType file_type,
+		CameraFileInfo * info, GPContext *context)
 {
 	unsigned int l = 0;
 	int rc;
 	Packet dp;
 
+	rc = sony_set_file_mode(camera, file_type);
+	if (rc != GP_OK) {
+		return rc;
+	}
+	
 	if (gp_context_cancel(context) == GP_CONTEXT_FEEDBACK_CANCEL) {
 		return GP_ERROR_CANCEL;
 	}
@@ -764,9 +883,15 @@ int sony_image_info(Camera * camera, int imageid, CameraFileInfo * info, GPConte
 
 		info->file.fields = GP_FILE_INFO_SIZE | GP_FILE_INFO_TYPE;
 		info->file.size = l;
-		strcpy (info->file.type, GP_MIME_JPEG);
 
 		info->preview.fields = GP_FILE_INFO_TYPE;
+
+		if (file_type == SONY_FILE_MPEG) {
+			strcpy (info->file.type, GP_MIME_AVI);
+		}
+		else {
+			strcpy (info->file.type, GP_MIME_JPEG);
+		}
 	}
 	return rc;
 }
