@@ -426,7 +426,12 @@ static int put_file_func (CameraFilesystem *fs, const char *folder,
     return GP_OK;
 }
 
-
+static CameraFilesystemFuncs fsfuncs = {
+	.file_list_func = file_list_func,
+	.get_file_func = get_file_func,
+	.put_file_func = put_file_func,
+	.del_file_func = delete_file_func,
+};
 
 int camera_init(Camera *camera, GPContext *context) {
    
@@ -496,16 +501,7 @@ int camera_init(Camera *camera, GPContext *context) {
 	camera->pl = NULL;
 	return (ret);
     }
-    
-       /* Tell the CameraFilesystem where to get lists from */
-    gp_filesystem_set_list_funcs (camera->fs, file_list_func, NULL, camera);
-    gp_filesystem_set_file_funcs (camera->fs, get_file_func, delete_file_func,
-		    		  camera);
-    gp_filesystem_set_folder_funcs (camera->fs, put_file_func,
-				    NULL, NULL, NULL, camera);
-   
-   
-
-    return GP_OK;
+    /* Tell the CameraFilesystem where to get lists from */
+    return gp_filesystem_set_funcs (camera->fs, &fsfuncs, camera);
 }
 
