@@ -141,14 +141,14 @@ int gpe_datsize(unsigned char *data,int tagind);
 int gpe_tagnum( char *data,int tagind);
 int gpe_getintval(unsigned char *data, int tagnum);
 
-long exif_get_lilend(unsigned char *data, int size);
+long gpi_exif_get_lilend(unsigned char *data, int size);
 int gpe_theval(unsigned char *data,int tagind);
 void gpe_setval(unsigned char *data,int tagind,long newval);
 long exif_next_ifd(unsigned char *exif,int num);
-int exif_get_comment(exifparser *exifdat, char *comment);
+int gpi_exif_get_comment(exifparser *exifdat, char *comment);
 int exif_set_comment(exifparser *exifdat, char *comment);
 int gpe_dump_ifd(int ifdnum,exifparser *exifdata,char **allpars);
-long exif_get_slilend(unsigned char *data, int size);
+long gpi_exif_get_slilend(unsigned char *data, int size);
 
 /*
  *  Utility functions: get fields in little-endian format,
@@ -163,7 +163,7 @@ long exif_get_slilend(unsigned char *data, int size);
  *
  * Returns : the value
  */
-long exif_get_lilend(unsigned char *data, int size){
+long gpi_exif_get_lilend(unsigned char *data, int size){
   long total;
 
   total=0;
@@ -178,7 +178,7 @@ long exif_get_lilend(unsigned char *data, int size){
 /*
  * Convert to  signed Intel-Endian number
  */
-long exif_get_slilend(unsigned char *data, int size){
+long gpi_exif_get_slilend(unsigned char *data, int size){
   long total;
   long unsigned mask=1<<(size*8-1);
 
@@ -198,7 +198,7 @@ long exif_get_slilend(unsigned char *data, int size){
  *  Return "value" of directory entry at tagind
  */
 int gpe_theval(unsigned char *data,int tagind){
- return(exif_get_lilend(data+tagind*12+10,4));
+ return(gpi_exif_get_lilend(data+tagind*12+10,4));
 }
 
 
@@ -226,7 +226,7 @@ long exif_next_ifd(unsigned char *exif,int num){
 
   int offset=(exif[num]+(exif[num+1]<<8))*12+num+2;
   if (exif_debug) printf("next_ifd,offset=%d\n",offset);
-  return(exif_get_lilend(exif+offset,4));
+  return(gpi_exif_get_lilend(exif+offset,4));
 
 }
 
@@ -269,27 +269,27 @@ int exif_parse_data(exifparser *exifdat) {
     return(-1);
   }
 
-  offset=exif_get_lilend(exifdat->data+4,4); /*Get offset of first IFD */
+  offset=gpi_exif_get_lilend(exifdat->data+4,4); /*Get offset of first IFD */
   exifdat->ifdcnt=-1;
 
   /*Step through each IFD */
   do {
     exifdat->ifdcnt++;
     exifdat->ifds[exifdat->ifdcnt]     = exifdat->data+offset;
-    exifdat->ifdtags[exifdat->ifdcnt]  = exif_get_lilend(exifdat->data+offset,2);
+    exifdat->ifdtags[exifdat->ifdcnt]  = gpi_exif_get_lilend(exifdat->data+offset,2);
   } while ((offset=exif_next_ifd(exifdat->data,offset)));
 
   exifdat->ifdcnt++;
 
   /* Look for the EXIF IFD */
   exifdat->preparsed=1; /* Indicate that our exifparser is initialized */
-  /* Has to be done before calling exif_get_field */
-  if ( ! exif_get_field( EXIF_ExifOffset, 0, exifdat, &tagdat)) {
+  /* Has to be done before calling gpi_exif_get_field */
+  if ( ! gpi_exif_get_field( EXIF_ExifOffset, 0, exifdat, &tagdat)) {
     printf("No EXIF information stored in this image\n");
   } else {
     if (exif_debug) printf("Offset to the Exif subIFD is %d\n",tagdat.intval);
     exifdat->ifds[exifdat->ifdcnt] = exifdat->data + tagdat.intval;
-    exifdat->ifdtags[exifdat->ifdcnt]  = exif_get_lilend(exifdat->data+tagdat.intval,2);
+    exifdat->ifdtags[exifdat->ifdcnt]  = gpi_exif_get_lilend(exifdat->data+tagdat.intval,2);
     exifdat->ifdcnt++;
   }
 
@@ -314,7 +314,7 @@ int exif_parse_data(exifparser *exifdat) {
  *
  * returns   : 1 on success, 0 on failure
  */
-int exif_get_field( int tag_number, int ifd, exifparser *exifdata, ExifData *tag_data) {
+int gpi_exif_get_field( int tag_number, int ifd, exifparser *exifdata, ExifData *tag_data) {
   int numtags,i, tag;
   char *ifdp, *data;
 
@@ -328,7 +328,7 @@ int exif_get_field( int tag_number, int ifd, exifparser *exifdata, ExifData *tag
   if (ifd == -1) {
     for (i = 0; i < exifdata->ifdcnt+1; i++){
       if (exif_debug) printf("Searching in IFD %d\n",i);
-      if ( exif_get_field(tag_number, i, exifdata, tag_data) == 1)
+      if ( gpi_exif_get_field(tag_number, i, exifdata, tag_data) == 1)
 	return 1;
     }
     return 0; /* We did not find it. */
@@ -336,12 +336,12 @@ int exif_get_field( int tag_number, int ifd, exifparser *exifdata, ExifData *tag
 
     /* Find tag in a specific IFD */
     ifdp = exifdata->ifds[ifd];
-    numtags=exif_get_lilend(ifdp,2); /* How many tags in the IFD */
-    if (exif_debug)  printf("exif_get_field: %d tags in ifd %d\n",numtags, ifd);
+    numtags=gpi_exif_get_lilend(ifdp,2); /* How many tags in the IFD */
+    if (exif_debug)  printf("gpi_exif_get_field: %d tags in ifd %d\n",numtags, ifd);
     i=-1;
     do {
       i++;
-      tag = exif_get_lilend(ifdp+i*12+2, 2); /* Get the tag ID */
+      tag = gpi_exif_get_lilend(ifdp+i*12+2, 2); /* Get the tag ID */
       /* if (exif_debug) fprintf(stderr,"Found tag %d \n",tag); */
     } while ((i < numtags) && (tag != tag_number));
     
@@ -352,8 +352,8 @@ int exif_get_field( int tag_number, int ifd, exifparser *exifdata, ExifData *tag
     
     ifdp = ifdp+i*12+2; /* Place pointer at TAG type position */
     tag_data->tag = tag;
-    tag_data->type = exif_get_lilend(ifdp+2,2);    /* tag type */
-    tag_data->size = exif_sizetab[tag_data->type]*exif_get_lilend(ifdp+4,4);
+    tag_data->type = gpi_exif_get_lilend(ifdp+2,2);    /* tag type */
+    tag_data->size = exif_sizetab[tag_data->type]*gpi_exif_get_lilend(ifdp+4,4);
     if (exif_debug) printf("(%d bytes) ",tag_data->size);
 
     ifdp += 8; /* place pointer at beginning of the contents of the field */
@@ -362,14 +362,14 @@ int exif_get_field( int tag_number, int ifd, exifparser *exifdata, ExifData *tag
        the IFD field value. Otherwise, that value is an offset
        to the real data */
     if (tag_data->size > 4) {
-	ifdp = exifdata->data + exif_get_lilend(ifdp,4);
+	ifdp = exifdata->data + gpi_exif_get_lilend(ifdp,4);
     }
 
     /* Last operation, get the data itself: */
     /* Allocate memory, and copy the data */
       data=malloc(tag_data->size);
       if (data==NULL){
-	fprintf(stderr,"exif_get_field: could not malloc\n");
+	fprintf(stderr,"gpi_exif_get_field: could not malloc\n");
 	return 0;
       }
 
@@ -383,8 +383,8 @@ int exif_get_field( int tag_number, int ifd, exifparser *exifdata, ExifData *tag
 	if ( tag_data->type % 5 ) {
 	  memcpy(data+i,ifdp+i,exif_sizetab[tag_data->type]);	  
 	} else { /* Fraction */
-	    tag_data->num =exif_get_lilend(ifdp+i,4);
-	    tag_data->den =exif_get_lilend(ifdp+i+4,4);
+	    tag_data->num =gpi_exif_get_lilend(ifdp+i,4);
+	    tag_data->den =gpi_exif_get_lilend(ifdp+i+4,4);
 	  
 	  if(exif_debug)  printf("%d/%d=%.3g ",tag_data->num,
 		                  tag_data->den,
@@ -395,7 +395,7 @@ int exif_get_field( int tag_number, int ifd, exifparser *exifdata, ExifData *tag
       /* it into "intval" right away... */
       if(tag_data->type != EXIF_ASCII && tag_data->type != EXIF_RATIONAL &&
 	 tag_data->type != EXIF_UNDEFINED && tag_data->type != EXIF_SRATIONAL) {
-	tag_data->intval = exif_get_lilend(data,exif_sizetab[tag_data->type]);
+	tag_data->intval = gpi_exif_get_lilend(data,exif_sizetab[tag_data->type]);
 	if(exif_debug) printf("'%d'",tag_data->intval);
       }
       tag_data->data = data; /* Save the data field */
@@ -417,12 +417,12 @@ int exif_get_field( int tag_number, int ifd, exifparser *exifdata, ExifData *tag
  *
  * returns   : pointer to the tag value.
  */
-char * exif_get_ascii_field( int tag_number, int ifd, exifparser *exifdat) {
+char * gpi_exif_get_ascii_field( int tag_number, int ifd, exifparser *exifdat) {
   ExifData tagdat;
 
   if (exif_parse_data(exifdat)<0) return NULL;
 
-  if ( ! exif_get_field( tag_number, ifd, exifdat, &tagdat)) {
+  if ( ! gpi_exif_get_field( tag_number, ifd, exifdat, &tagdat)) {
     if (exif_debug) printf("No comment field in this image\n");
     return NULL;
   } else {
@@ -442,13 +442,13 @@ char * exif_get_ascii_field( int tag_number, int ifd, exifparser *exifdat) {
  *
  * returns   : tag value.
  */
-int exif_get_int_field( int tag_number, int ifd, exifparser *exifdat) {
+int gpi_exif_get_int_field( int tag_number, int ifd, exifparser *exifdat) {
   ExifData tagdat;
   int tmp;
 
   if (exif_parse_data(exifdat)<0) return 0;
 
-  if ( ! exif_get_field( tag_number, ifd, exifdat, &tagdat)) {
+  if ( ! gpi_exif_get_field( tag_number, ifd, exifdat, &tagdat)) {
     if (exif_debug) printf("Field not found in this image.\n");
     return 0;
   } else {
@@ -462,7 +462,7 @@ int exif_get_int_field( int tag_number, int ifd, exifparser *exifdat) {
 /*
  * Returns the name of a given tag number
  */
-char *exif_get_tagname(int tag_number) {
+char *gpi_exif_get_tagname(int tag_number) {
   int i=-1;
   do {
     i++;
@@ -479,13 +479,13 @@ char *exif_get_tagname(int tag_number) {
 /*
  * Gets the comment field if it exists.
  */
-int exif_get_comment(exifparser *exifdat, char *comment) {
+int gpi_exif_get_comment(exifparser *exifdat, char *comment) {
   ExifData tagdat;
 
   if (exif_parse_data(exifdat)<0) return 0;
 
   /* User Comment is in IFD number 2 */
-  if ( ! exif_get_field( EXIF_UserComment, 2, exifdat, &tagdat)) {
+  if ( ! gpi_exif_get_field( EXIF_UserComment, 2, exifdat, &tagdat)) {
     if (exif_debug) printf("No comment field in this image\n");
     return 0;
   } else {
@@ -506,11 +506,11 @@ int exif_set_comment(exifparser *exifdat, char *comment) {
  *
  * The returned thumbnail might be in JPEG or TIFF format.
  */
-unsigned char *exif_get_thumbnail(exifparser *exifdat) {
+unsigned char *gpi_exif_get_thumbnail(exifparser *exifdat) {
 
    long size;
 
-   return exif_get_thumbnail_and_size(exifdat, &size);
+   return gpi_exif_get_thumbnail_and_size(exifdat, &size);
 }
 
 
@@ -520,7 +520,7 @@ unsigned char *exif_get_thumbnail(exifparser *exifdat) {
  *
  * The returned thumbnail might be in JPEG or TIFF format.
  */
-unsigned char *exif_get_thumbnail_and_size(exifparser *exifdat, long *size) {
+unsigned char *gpi_exif_get_thumbnail_and_size(exifparser *exifdat, long *size) {
   unsigned char *imagedata,*exifimg,*newimg,*curptr;
   unsigned int entry;
   long dataptr,dsize,tag,datvec,tmp;
@@ -532,7 +532,7 @@ unsigned char *exif_get_thumbnail_and_size(exifparser *exifdat, long *size) {
   *size = 0;
   newimg=malloc(exifdat->exiflen);
   if (newimg==NULL){
-    fprintf(stderr,"exif_get_thumbnail: could not malloc\n");
+    fprintf(stderr,"gpi_exif_get_thumbnail: could not malloc\n");
     return(NULL);
   }
 
@@ -545,11 +545,11 @@ unsigned char *exif_get_thumbnail_and_size(exifparser *exifdat, long *size) {
       ExifData owner;
       char *comment=NULL;
       printf("Decoding EXIF fields in thumbnail\n");
-      exif_get_field( EXIF_Model, -1, exifdat, &owner);
+      gpi_exif_get_field( EXIF_Model, -1, exifdat, &owner);
       printf("Camera model: %s\n",owner.data);
-      printf("Comment for this picture (%d chars)",exif_get_comment( exifdat, comment));
+      printf("Comment for this picture (%d chars)",gpi_exif_get_comment( exifdat, comment));
       /*printf(" -> %s\n",comment); Not OK on Linux */
-      exif_get_field( EXIF_SubjectDistance, 2, exifdat, &owner);
+      gpi_exif_get_field( EXIF_SubjectDistance, 2, exifdat, &owner);
       /*      dump_exif(exifdat);       */
   }
 
@@ -570,7 +570,7 @@ unsigned char *exif_get_thumbnail_and_size(exifparser *exifdat, long *size) {
   curptr+=2;
   *size += 2;
 
-  entry=exif_get_lilend(exifimg,2);
+  entry=gpi_exif_get_lilend(exifimg,2);
 
   if (exif_debug) printf("Entry is %d \n",entry);
 
@@ -656,7 +656,7 @@ unsigned char *exif_get_thumbnail_and_size(exifparser *exifdat, long *size) {
  * returns: tag ID number
  */
 int gpe_tagnum( char *data,int tagind){
- return(exif_get_lilend(data+tagind*12+2,2));
+ return(gpi_exif_get_lilend(data+tagind*12+2,2));
 }
 
 /*
@@ -671,14 +671,14 @@ int gpe_tagnum( char *data,int tagind){
 int gpe_getintval(unsigned char *data, int tagnum) {
   int numtags,i,tag,tagtype;
 
-  numtags=exif_get_lilend(data,2);
+  numtags=gpi_exif_get_lilend(data,2);
 
   if (exif_debug)  printf("getval:%d tags\n",numtags);
 
  i=-1;
  do {
    i++;
-   tag=exif_get_lilend(data+i*12+2,2);
+   tag=gpi_exif_get_lilend(data+i*12+2,2);
  } while ((i<numtags) && (tag!=tagnum));
 
  if(tag!=tagnum) {
@@ -686,9 +686,9 @@ int gpe_getintval(unsigned char *data, int tagnum) {
    return(-1);
  }
 
- tagtype=exif_get_lilend(data+i*12+4,2);    /* tag type */
+ tagtype=gpi_exif_get_lilend(data+i*12+4,2);    /* tag type */
 
- return(exif_get_lilend(data+i*12+10,exif_sizetab[tagtype-1]));
+ return(gpi_exif_get_lilend(data+i*12+10,exif_sizetab[tagtype-1]));
 }
 
 /**
@@ -698,7 +698,7 @@ int gpe_getintval(unsigned char *data, int tagnum) {
  * tagind: tag index
  */
 int gpe_getvalue(unsigned char *data,int tagind){
- return(exif_get_lilend(data+tagind*12+10,4));
+ return(gpi_exif_get_lilend(data+tagind*12+10,4));
 }
 
 
@@ -725,24 +725,24 @@ int gpe_dump_ifd(int ifdnum,exifparser *exifdata,char **allpars){
   
   
     thisisd=exifdata->ifds[ifdnum];
-    numtags=exif_get_lilend(thisisd,2);
+    numtags=gpi_exif_get_lilend(thisisd,2);
     printf("has %d tags ----------------------\n",numtags);
     for (i=0;i<numtags;i++){
 
       thistag=thisisd+i*12+2;   /* pointer to data for this tag */
 
-      tag=exif_get_lilend(thistag,2);          /* tag identifier */
-      tagtype=exif_get_lilend(thistag+2,2);    /* tag type */
-      count = exif_get_lilend(thistag+4, 4);   /* how many */
+      tag=gpi_exif_get_lilend(thistag,2);          /* tag identifier */
+      tagtype=gpi_exif_get_lilend(thistag+2,2);    /* tag type */
+      count = gpi_exif_get_lilend(thistag+4, 4);   /* how many */
       typelen=exif_sizetab[tagtype];/* length of this type */
 
       if (exif_debug) printf("(%dX) ",count);
 
       thedata=thistag+8;
       if (count*typelen > 4)   /* find it in a data block elsewhere */
-	thedata = exifdata->data+exif_get_lilend(thistag+8, 4);
+	thedata = exifdata->data+gpi_exif_get_lilend(thistag+8, 4);
 
-      printf("Tag 0x%X %s = ",tag,exif_get_tagname(tag));
+      printf("Tag 0x%X %s = ",tag,gpi_exif_get_tagname(tag));
 
       if (tagtype==2) {
 	/* Do an ASCII tag */
@@ -753,12 +753,12 @@ int gpe_dump_ifd(int ifdnum,exifparser *exifdata,char **allpars){
       else for (j=0;j<count;j++) {
 
 	if ((tagtype==5)||(tagtype==10)) {/* Fractional */
-	  tmp1=exif_get_slilend(thedata+j*typelen,4);
-	  tmp2=exif_get_slilend(thedata+4+j*typelen,4);
+	  tmp1=gpi_exif_get_slilend(thedata+j*typelen,4);
+	  tmp2=gpi_exif_get_slilend(thedata+4+j*typelen,4);
 	      printf("%d/%d=%.3g ",tmp1,tmp2,(tmp2==0)?0:(1.0*tmp1/tmp2));
 	}
 	else {
-	  value =exif_get_lilend(thedata+j*typelen,typelen);
+	  value =gpi_exif_get_lilend(thedata+j*typelen,typelen);
 	  printf("%d ",value);
 	}
 	
@@ -771,7 +771,7 @@ int gpe_dump_ifd(int ifdnum,exifparser *exifdata,char **allpars){
          printf("Exif SubIFD at offset %d\n", value );
          exifdata->ifds[exifdata->ifdcnt]     = exifdata->data+value;  
 /*       exifdata->ifds[exifdata->ifdcnt]; */
-         exifdata->ifdtags[exifdata->ifdcnt]=exif_get_lilend(exifdata->data+value,2);
+         exifdata->ifdtags[exifdata->ifdcnt]=gpi_exif_get_lilend(exifdata->data+value,2);
          exifdata->ifdcnt++;
 
 }
@@ -784,11 +784,11 @@ int gpe_dump_ifd(int ifdnum,exifparser *exifdata,char **allpars){
 
 
 /* Walk through EXIF structure, printing values */
-int gpe_dump_exif(exifparser *exifdata){ 
+int gpi_exif_dump(exifparser *exifdata){ 
   int i;
 
   if (!exifdata->preparsed) 
-    if (stat_exif(exifdata)) return(-1);
+    if (gpi_exif_stat(exifdata)) return(-1);
 
   for (i=0;i<exifdata->ifdcnt;i++){
     switch (i) {
@@ -814,10 +814,10 @@ int gpe_dump_exif(exifparser *exifdata){
  * Return data size of directory entry at tagind
  */
 int gpe_datsize(unsigned char *data,int tagind){
-  return(exif_sizetab[exif_get_lilend(data+tagind*12+4,2)]*exif_get_lilend(data+tagind*12+6,4));
+  return(exif_sizetab[gpi_exif_get_lilend(data+tagind*12+4,2)]*gpi_exif_get_lilend(data+tagind*12+6,4));
 }
 
-int stat_exif(exifparser *exifdata) {
+int gpi_exif_stat(exifparser *exifdata) {
   long offset=0;
 
   exifdata->endian=0;
@@ -828,7 +828,7 @@ int stat_exif(exifparser *exifdata) {
     return(-1);
   }
 
-  offset=exif_get_lilend(exifdata->data+4,4); /*Get offset of first IFD*/
+  offset=gpi_exif_get_lilend(exifdata->data+4,4); /*Get offset of first IFD*/
 
   exifdata->ifdcnt=-1;
 
@@ -837,7 +837,7 @@ int stat_exif(exifparser *exifdata) {
     exifdata->ifdcnt++;
     exifdata->ifds[exifdata->ifdcnt]     = exifdata->data+offset;
     exifdata->ifdtags[exifdata->ifdcnt]  = 
-    exif_get_lilend(exifdata->data+offset,2);
+    gpi_exif_get_lilend(exifdata->data+offset,2);
 
   }while((offset=exif_next_ifd(exifdata->data,offset)));
   exifdata->ifdcnt++;
