@@ -1,17 +1,21 @@
-/* gphoto2-abilities-list.c
+/** \file gphoto2-abilities-list.c
+ * \brief List of supported camera models including their abilities.
  *
- * Copyright  2000 Scott Fritzinger
+ * \author Copyright  2000 Scott Fritzinger
  *
+ * \par
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
+ * \par
  * This library is distributed in the hope that it will be useful, 
  * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details. 
  *
+ * \par
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
@@ -25,9 +29,7 @@
 #include <stdio.h>
 #include <string.h>
 
-#ifdef HAVE_LTDL
-#  include <ltdl.h>
-#endif
+#include <ltdl.h>
 
 #include "gphoto2-result.h"
 #include "gphoto2-port-log.h"
@@ -48,22 +50,31 @@
 #  define dgettext(Domain,Message) (Message)
 #  define dcgettext(Domain,Message,Type) (Message)
 #  define bindtextdomain(Domain,Directory) (Domain)
+#  define bind_textdomain_codeset(Domain,Charset) (Domain)
 #  define _(String) (String)
 #  define N_(String) (String)
 #endif
 
+/** \internal */
 #define GP_MODULE "gphoto2-abilities-list"
 
+/** \internal */
 #define CHECK_NULL(r)        {if (!(r)) return (GP_ERROR_BAD_PARAMETERS);}
+/** \internal */
 #define CHECK_RESULT(result) {int r = (result); if (r < 0) return (r);}
+/** \internal */
 #define CHECK_MEM(m)         {if (!(m)) return (GP_ERROR_NO_MEMORY);}
 
 /**
- * CameraAbilitiesList:
+ * Name of the environment variable which may contain the path where
+ * to look for the camlibs. If this environment variable is not defined,
+ * use the compiled-in default constant.
  *
- * A list of supported camera models including their abilities. The internals
- * of this list are hidden - please use functions to access the list.
- **/
+ * \internal Internal use only.
+ */
+#define CAMLIBDIR_ENV "CAMLIBS"
+
+/** \internal */
 struct _CameraAbilitiesList {
 	int count;
 	CameraAbilities *abilities;
@@ -73,14 +84,28 @@ static int gp_abilities_list_lookup_id (CameraAbilitiesList *, const char *);
 static int gp_abilities_list_sort      (CameraAbilitiesList *);
 
 /**
- * gp_abilities_list_new:
- * @list:
+ * Set the codeset for all messages returned by libgphoto2.
+ * \param codeset New codeset for the messages. For instance "utf-8".
+ * \return old codeset as returned from bind_textdomain_codeset().
  *
- * Allocates the memory for a new abilities list. You would then call
- * #gp_abilities_list_load in order to populate it.
+ * You would then call gp_abilities_list_load() in order to 
+ * populate it.
+ */
+const char*
+gp_message_codeset (const char *codeset)
+{
+	gp_port_message_codeset (codeset);
+	return bind_textdomain_codeset (GETTEXT_PACKAGE, codeset);
+}
+
+/**
+ * Allocate the memory for a new abilities list.
+ * \param list CameraAbilitiesList object to initialize
+ * \return gphoto2 error code
  *
- * Return value: a gphoto2 error code
- **/
+ * You would then call gp_abilities_list_load() in order to 
+ * populate it.
+ */
 int
 gp_abilities_list_new (CameraAbilitiesList **list)
 {
@@ -101,13 +126,11 @@ gp_abilities_list_new (CameraAbilitiesList **list)
 }
 
 /**
- * gp_abilities_list_free:
- * @list: a #CameraAbilitiesList
+ * \brief Free the given CameraAbilitiesList object.
  *
- * Frees the @list.
- *
- * Return value: a gphoto2 error code
- **/
+ * \param list a CameraAbilitiesList
+ * \return a gphoto2 error code
+ */
 int
 gp_abilities_list_free (CameraAbilitiesList *list)
 {
@@ -120,20 +143,20 @@ gp_abilities_list_free (CameraAbilitiesList *list)
 	return (GP_OK);
 }
 
-#ifdef HAVE_LTDL
 
 static int
 foreach_func (const char *filename, lt_ptr data)
 {
 	CameraList *list = data;
+	int result;
 
 	gp_log (GP_LOG_DEBUG, "gphoto2-abilities-list",
 		"Found '%s'.", filename);
-	gp_list_append (list, filename, NULL);
+	result = gp_list_append (list, filename, NULL);
 
-	return (0);
+	return ((result == GP_OK)?0:1);
 }
-#endif
+
 
 static int
 gp_abilities_list_load_dir (CameraAbilitiesList *list, const char *dir,
@@ -145,29 +168,29 @@ gp_abilities_list_load_dir (CameraAbilitiesList *list, const char *dir,
 	int x, old_count, new_count;
 	unsigned int i, p;
 	const char *filename;
-#ifdef HAVE_LTDL
 	CameraList *flist;
 	int count;
+	int ret;
 	lt_dlhandle lh;
-#else
-	GP_SYSTEM_DIR d;
-	GP_SYSTEM_DIRENT de;
-	char buf[1024];
-	void *lh;
-	unsigned int n;
-#endif
 
 	CHECK_NULL (list && dir);
 
-#ifdef HAVE_LTDL
 	gp_log (GP_LOG_DEBUG, "gphoto2-abilities-list",
-		"Using ltdl to load camera libraries in '%s'...", dir);
+		"Using ltdl to load camera libraries from '%s'...", dir);
 	CHECK_RESULT (gp_list_new (&flist));
 	CHECK_RESULT (gp_list_reset (flist));
 	lt_dlinit ();
 	lt_dladdsearchdir (dir);
-	lt_dlforeachfile (dir, foreach_func, flist);
+	ret = lt_dlforeachfile (dir, foreach_func, flist);
 	lt_dlexit ();
+	if (ret != 0) {
+		gp_log (GP_LOG_ERROR, "gp-abilities-list", 
+			"Internal error looking for camlibs (%d)", ret);
+		gp_context_error (context,
+				  _("Internal error looking for camlibs. "
+				    "(path names too long?)"));
+		return (GP_ERROR);
+	}
 	CHECK_RESULT (count = gp_list_count (flist));
 	gp_log (GP_LOG_DEBUG, "gp-abilities-list", "Found %i "
 		"camera drivers.", count);
@@ -252,162 +275,35 @@ gp_abilities_list_load_dir (CameraAbilitiesList *list, const char *dir,
 	gp_context_progress_stop (context, p);
 	lt_dlexit ();
 	gp_list_free (flist);
-#else
-
-	gp_log (GP_LOG_DEBUG, "gphoto2-abilities-list",
-		"Loading camera libraries in '%s' without ltdl...", dir);
-	gp_log (GP_LOG_DEBUG, "gphoto2-abilities-list",
-		"Note that failing to load *.a and *.la is NOT an error!");
-	/* Open the directory */
-	d = GP_SYSTEM_OPENDIR (dir);
-	if (!d) {
-		gp_log (GP_LOG_ERROR, "gphoto2-abilities-list",
-			_("Could not open '%s'"), dir);
-		return GP_ERROR_LIBRARY;
-	}
-
-	/* Count the files */
-	n = 0;
-	while (GP_SYSTEM_READDIR (d))
-		n++;
-	GP_SYSTEM_CLOSEDIR (d);
-	d = GP_SYSTEM_OPENDIR (dir);
-
-	p = gp_context_progress_start (context, n,
-			_("Loading camera drivers from '%s'..."), dir);
-	i = 0;
-	do {
-
-		/* Read each entry */
-		de = GP_SYSTEM_READDIR (d);
-		if (de) {
-
-			/*
-			 * Tell the frontend about our progress and offer the
-			 * possiblility to cancel.
-			 */
-			i++;
-			gp_context_progress_update (context, p, i);
-			if (gp_context_cancel (context) ==
-						GP_CONTEXT_FEEDBACK_CANCEL)
-				return (GP_ERROR_CANCEL);
-
-			/* Construct the full path to the file */
-			filename = GP_SYSTEM_FILENAME (de);
-			snprintf (buf, sizeof (buf), "%s%c%s", dir,
-				  GP_SYSTEM_DIR_DELIM, filename);
-
-			/* Don't try to open ".*" */
-			if (filename[0] == '.')
-				continue;
-
-			/* Try to open the library */
-			gp_log (GP_LOG_DEBUG, "gphoto2-abilities-list",
-				"Trying to load '%s'...", buf);
-			lh = GP_SYSTEM_DLOPEN (buf);
-			if (!lh) {
-				size_t len;
-				len = strlen(buf);
-				if ((len >= 3) &&
-				    (buf[len-1] == 'a') &&
-				    ((buf[len-2] == '.') ||
-				     ((buf[len-2] == 'l') && (buf[len-3] == '.'))
-					    )) {
-					/* *.la or *.a - we cannot load these, so no error msg */
-				} else {
-					gp_log (GP_LOG_DEBUG, "gphoto2-abilities-list",
-						"Failed to load '%s': %s.", buf,
-						GP_SYSTEM_DLERROR ());
-				}
-				continue;
-			}
-
-			/* camera_id */
-			id = GP_SYSTEM_DLSYM (lh, "camera_id");
-			if (!id) {
-				gp_log (GP_LOG_DEBUG, "gphoto2-abilities-list",
-					"Library '%s' does not seem to "
-					"contain a camera_id function: %s",
-					buf, GP_SYSTEM_DLERROR ());
-				GP_SYSTEM_DLCLOSE (lh);
-				continue;
-			}
-
-			/*
-			 * Make sure the camera driver hasn't been
-			 * loaded yet
-			 */
-			if (id (&text) != GP_OK) {
-				GP_SYSTEM_DLCLOSE (lh);
-				continue;
-			}
-			if (gp_abilities_list_lookup_id (list, text.text) >= 0){
-				GP_SYSTEM_DLCLOSE (lh);
-				continue;
-			}
-
-			/* camera_abilities */
-			ab = GP_SYSTEM_DLSYM (lh, "camera_abilities");
-			if (!ab) {
-				gp_log (GP_LOG_DEBUG, "gphoto2-abilities-list",
-					"Library '%s' does not seem to "
-					"contain a camera_abilities function: "
-					"%s", buf, GP_SYSTEM_DLERROR ());
-				GP_SYSTEM_DLCLOSE (lh);
-				continue;
-			}
-
-			old_count = gp_abilities_list_count (list);
-			if (old_count < 0) {
-				GP_SYSTEM_DLCLOSE (lh);
-				continue;
-			}
-
-			if (ab (list) != GP_OK) {
-				GP_SYSTEM_DLCLOSE (lh);
-				continue;
-			}
-
-			GP_SYSTEM_DLCLOSE (lh);
-
-			new_count = gp_abilities_list_count (list);
-			if (new_count < 0)
-				continue;
-
-			/* Copy in the core-specific information */
-			for (x = old_count; x < new_count; x++) {
-				strcpy (list->abilities[x].id, text.text);
-				strcpy (list->abilities[x].library, buf);
-			}
-		}
-	} while (de);
-	gp_context_progress_stop (context, p);
-#endif
 
 	return (GP_OK);
 }
 
+
 /**
- * gp_abilities_list_load:
- * @list: a #CameraAbilitiesList
- * @context: a #GPContext
+ * \brief Scans the system for camera drivers.
  *
- * Scans the system for camera drivers. All supported camera models will then
- * be added to the @list.
+ * \param list a CameraAbilitiesList
+ * \param context a GPContext
+ * \return a gphoto2 error code
  *
- * Return value: a gphoto2 error code
- **/
+ * All supported camera models will then be added to the list.
+ *
+ */
 int
 gp_abilities_list_load (CameraAbilitiesList *list, GPContext *context)
 {
+	const char *camlib_env = getenv(CAMLIBDIR_ENV);
+	const char *camlibs = (camlib_env != NULL)?camlib_env:CAMLIBS;
 	CHECK_NULL (list);
 
-	CHECK_RESULT (gp_abilities_list_load_dir (list, CAMLIBS, context));
+	CHECK_RESULT (gp_abilities_list_load_dir (list, camlibs, context));
 	CHECK_RESULT (gp_abilities_list_sort (list));
 
 	return (GP_OK);
 }
 
+
 static int
 gp_abilities_list_detect_usb (CameraAbilitiesList *list,
 			      int *ability, GPPort *port)
@@ -482,17 +378,17 @@ gp_abilities_list_detect_usb (CameraAbilitiesList *list,
 	return res;
 }
 
+
 /**
- * gp_abilities_list_detect:
- * @list: a #CameraAbilitiesList
- * @info_list: a #GPPortInfoList
- * @l: a #CameraList
+ * \param list a CameraAbilitiesList
+ * \param info_list a GPPortInfoList
+ * \param l a CameraList
+ * \return a gphoto2 error code
  *
  * Tries to detect any camera connected to the computer using the supplied
- * @list of supported cameras and the supplied @info_list of ports.
+ * list of supported cameras and the supplied info_list of ports.
  *
- * Return value: a gphoto2 error code
- **/
+ */
 int
 gp_abilities_list_detect (CameraAbilitiesList *list,
 			  GPPortInfoList *info_list, CameraList *l,
@@ -500,11 +396,11 @@ gp_abilities_list_detect (CameraAbilitiesList *list,
 {
 	GPPortInfo info;
 	GPPort *port;
-	int i, info_count, ability;
+	int i, info_count;
 
 	CHECK_NULL (list && info_list && l);
 
-	l->count = 0;
+	gp_list_reset (l);
 
 	CHECK_RESULT (info_count = gp_port_info_list_count (info_list));
 
@@ -515,7 +411,8 @@ gp_abilities_list_detect (CameraAbilitiesList *list,
 		CHECK_RESULT (gp_port_info_list_get_info (info_list, i, &info));
 		CHECK_RESULT (gp_port_set_info (port, info));
 		switch (info.type) {
-		case GP_PORT_USB:
+		case GP_PORT_USB: {
+			int ability;
 			res = gp_abilities_list_detect_usb (list, &ability, port);
 			if (res == GP_OK) {
 				gp_list_append(l,
@@ -525,7 +422,34 @@ gp_abilities_list_detect (CameraAbilitiesList *list,
 				gp_port_set_error (port, NULL);
 
 			break;
+		}
+		case GP_PORT_DISK: {
+			char	*s, path[1024];
+			struct stat stbuf;
+		
+			s = strchr (info.path, ':');
+			if (!s)
+				break;
+			s++;
+			snprintf (path, sizeof(path), "%s/DCIM", s);
+			if (-1 == stat(path, &stbuf)) {
+				snprintf (path, sizeof(path), "%s/dcim", s);
+				if (-1 == stat(path, &stbuf))
+					continue;
+			}
+			gp_list_append (l, "Mass Storage Camera", info.path);
+			break;
+		}
+		case GP_PORT_PTPIP: {
+			char	*s;
 
+			s = strchr (info.path, ':');
+			if (!s) break;
+			s++;
+			if (!strlen(s)) break;
+			gp_list_append (l, "PTP/IP Camera", info.path);
+			break;
+		}
 		default:
 			/*
 			 * We currently cannot detect any cameras on this
@@ -542,11 +466,9 @@ gp_abilities_list_detect (CameraAbilitiesList *list,
 
 
 /**
- * remove_colon_from_string:
- * @str: a char * string
- *
- * Remove first colon from string, if any. Replace it by a space.
- **/
+ * \brief Remove first colon from string, if any. Replace it by a space.
+ * \param str a char * string
+ */
 static void
 remove_colon_from_string (char *str)
 {
@@ -559,16 +481,15 @@ remove_colon_from_string (char *str)
 
 
 /**
- * gp_abilities_list_append:
- * @list: a #CameraAbilitiesList
- * @abilities: #CameraAbilities
+ * \brief Append the abilities to the list.
+ * \param list  CameraAbilitiesList
+ * \param abilities  CameraAbilities
+ * \return a gphoto2 error code
  *
- * Appends the @abilities to the @list. This function is called by a camera
- * library on #camera_abilities in order to inform gphoto2 about a supported
- * camera model.
+ * This function is called by a camera library on camera_abilities()
+ * in order to inform libgphoto2 about a supported camera model.
  *
- * Return value: a gphoto2 error code
- **/
+ */
 int
 gp_abilities_list_append (CameraAbilitiesList *list, CameraAbilities abilities)
 {
@@ -598,14 +519,12 @@ gp_abilities_list_append (CameraAbilitiesList *list, CameraAbilities abilities)
 	return (GP_OK);
 }
 
+
 /**
- * gp_abilities_list_reset:
- * @list: a #CameraAbilitiesList
- *
- * Resets the list.
- *
- * Return value: a gphoto2 error code
- **/
+ * \brief Reset the list.
+ * \param list a CameraAbilitiesList
+ * \return a gphoto2 error code
+ */
 int
 gp_abilities_list_reset (CameraAbilitiesList *list)
 {
@@ -620,14 +539,12 @@ gp_abilities_list_reset (CameraAbilitiesList *list)
 	return (GP_OK);
 }
 
+
 /**
- * gp_abilities_list_count:
- * @list: a #CameraAbilitiesList
- *
- * Counts the entries in the supplied @list.
- *
- * Return value: The number of entries or a gphoto2 error code
- **/
+ * \brief Count the entries in the supplied list.
+ * \param a CameraAbilitiesList
+ * \return The number of entries or a gphoto2 error code
+ */
 int
 gp_abilities_list_count (CameraAbilitiesList *list)
 {
@@ -636,6 +553,7 @@ gp_abilities_list_count (CameraAbilitiesList *list)
 	return (list->count);
 }
 
+
 static int
 gp_abilities_list_sort (CameraAbilitiesList *list)
 {
@@ -660,6 +578,7 @@ gp_abilities_list_sort (CameraAbilitiesList *list)
 	return (GP_OK);
 }
 
+
 static int
 gp_abilities_list_lookup_id (CameraAbilitiesList *list, const char *id)
 {
@@ -674,15 +593,13 @@ gp_abilities_list_lookup_id (CameraAbilitiesList *list, const char *id)
 	return (GP_ERROR);
 }
 
+
 /**
- * gp_abilities_list_lookup_model
- * @list: a #CameraAbilitiesList
- * @model: a camera model
- *
- * Searches the @list for an entry of given @model.
- *
- * Return value: Index of entry or gphoto2 error code
- **/
+ * \brief Search the list for an entry of given model name
+ * \param list a #CameraAbilitiesList
+ * \param model a camera model name
+ * \return Index of entry or gphoto2 error code
+ */
 int
 gp_abilities_list_lookup_model (CameraAbilitiesList *list, const char *model)
 {
@@ -700,18 +617,18 @@ gp_abilities_list_lookup_model (CameraAbilitiesList *list, const char *model)
 	return (GP_ERROR_MODEL_NOT_FOUND);
 }
 
+
 /**
- * gp_abilities_list_get_abilities:
- * @list: a #CameraAbilitiesList
- * @index: index
- * @abilities: #CameraAbilities
+ * \brief Retrieve the camera abilities of entry with supplied index number.
+ * \param list a CameraAbilitiesList
+ * \param index index
+ * \param abilities CameraAbilities
+ * \return a gphoto2 error code
  *
- * Retrieves the camera @abilities of entry with supplied @index. Typically,
- * you would call #gp_camera_set_abilities afterwards in order to prepare the
- * initialization of a camera.
- *
- * Return value: a gphoto2 error code
- **/
+ * Retrieves the camera @abilities of entry with supplied
+ * index number. Typically, you would call gp_camera_set_abilities()
+ * afterwards in order to prepare the initialization of a camera.
+ */
 int
 gp_abilities_list_get_abilities (CameraAbilitiesList *list, int index,
 				 CameraAbilities *abilities)
@@ -726,6 +643,7 @@ gp_abilities_list_get_abilities (CameraAbilitiesList *list, int index,
 	return (GP_OK);
 }
 
+
 /*
  * Local Variables:
  * c-file-style:"linux"
