@@ -32,26 +32,11 @@
 #include "gphoto2-result.h"
 #include "gphoto2-file.h"
 #include "gphoto2-port-log.h"
+#include "gphoto2-setting.h"
 
-#ifdef HAVE_PROCMEMINFO
-#  include <fcntl.h>
-#elif (defined(sun) || defined(__sun__)) && defined(__svr4__)
-#  include <sys/stat.h>
-#  include <sys/swap.h>
-#  include <sys/fcntl.h>
-#endif
-
-#ifdef HAVE_SYSCTL
-#	if __FreeBSD__
-#		include <sys/types.h>
-#	elif (__NetBSD__ || __OpenBSD__)
-#		include <sys/param.h>
-#	endif
-#	include <sys/sysctl.h>
-#endif
 #include <limits.h>
 
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 #  include <libexif/exif-data.h>
 #endif
 
@@ -76,8 +61,12 @@
 
 #define GP_MODULE "libgphoto2"
 
+#ifndef PATH_MAX
+# define PATH_MAX 4096
+#endif
+
 typedef struct _CameraFilesystemFile {
-	char name [128];
+	char name [256];
 	int info_dirty;
 	CameraFileInfo info;
 	struct _CameraFilesystemFile *lru_prev;
@@ -87,17 +76,24 @@ typedef struct _CameraFilesystemFile {
 	CameraFile *raw;
 	CameraFile *audio;
 	CameraFile *exif;
+	CameraFile *metadata;
 } CameraFilesystemFile;
 
 typedef struct {
 	int count;
-	char name [128];
+	char name [PATH_MAX];
 	int files_dirty;
 	int folders_dirty;
 	CameraFilesystemFile *file;
 } CameraFilesystemFolder;
 
-#ifdef HAVE_EXIF
+#define PICTURES_TO_KEEP	2
+static int pictures_to_keep = -1;
+
+static int gp_filesystem_lru_clear (CameraFilesystem *fs);
+static int gp_filesystem_lru_remove_one (CameraFilesystem *fs, CameraFilesystemFile *item);
+
+#ifdef HAVE_LIBEXIF
 
 static int gp_filesystem_get_file_impl (CameraFilesystem *, const char *,
 		const char *, CameraFileType, CameraFile *, GPContext *);
@@ -133,7 +129,7 @@ get_exif_mtime (const unsigned char *data, unsigned long size)
         /*
          * HP PhotoSmart C30 has the date and time in ifd_exif.
          */
-#ifdef HAVE_EXIF_0_5_4
+#ifdef HAVE_LIBEXIF_IFD
 	e = exif_content_get_entry (ed->ifd[EXIF_IFD_0], EXIF_TAG_DATE_TIME);
 	if (e)
 		t1 = get_time_from_exif_tag(e);
@@ -291,7 +287,7 @@ struct _CameraFilesystem {
 	if (bufsize <= string_len) { \
 		GP_DEBUG ("%s: strlen(...) = %d " \
 			">= sizeof(buffer) = %d", \
-			msg, string_len, bufsize \
+			msg, (int)string_len, (int)bufsize \
 		); \
 		gp_context_error (context, "preventing buffer overflow"); \
 		return GP_ERROR; \
@@ -305,9 +301,9 @@ delete_all_files (CameraFilesystem *fs, int x)
         CHECK_NULL (fs);
 
         if (fs->folder[x].count) {
-
 		/* Get rid of cached files */
                 for (y = 0; y < fs->folder[x].count; y++) {
+			gp_filesystem_lru_remove_one (fs, &fs->folder[x].file[y]);
 			if (fs->folder[x].file[y].preview) {
 	                        gp_file_unref (fs->folder[x].file[y].preview);
 				fs->folder[x].file[y].preview = NULL;
@@ -328,6 +324,10 @@ delete_all_files (CameraFilesystem *fs, int x)
 				gp_file_unref (fs->folder[x].file[y].exif);
 				fs->folder[x].file[y].exif = NULL;
 			}
+			if (fs->folder[x].file[y].metadata) {
+				gp_file_unref (fs->folder[x].file[y].metadata);
+				fs->folder[x].file[y].metadata = NULL;
+			}
 		}
 
                 free (fs->folder[x].file);
@@ -410,8 +410,7 @@ append_folder (CameraFilesystem *fs, const char *folder, GPContext *context)
 
         /* Make sure the directory doesn't exist */
 	for (x = 0; x < fs->count; x++)
-		if (!strncmp (fs->folder[x].name, folder, strlen (folder)) &&
-		    (strlen (fs->folder[x].name) == strlen (folder)))
+		if (!strcmp (fs->folder[x].name, folder))
 			break;
 	if (x < fs->count) {
 		gp_context_error (context, _("Could not append folder '%s' "
@@ -431,7 +430,7 @@ append_folder (CameraFilesystem *fs, const char *folder, GPContext *context)
         if (x > 0) {
                 buf[x] = '\0';
 		for (x = 0; x < fs->count; x++)
-			if (!strncmp (fs->folder[x].name, buf, strlen (buf)))
+			if (!strcmp (fs->folder[x].name, buf))
 				break;
 		if (x == fs->count)
 			CR (append_folder (fs, buf, context))
@@ -481,8 +480,8 @@ append_file (CameraFilesystem *fs, int x, CameraFile *file, GPContext *context)
 	fs->folder[x].count++;
 	memset (&(fs->folder[x].file[fs->folder[x].count - 1]), 0,
 		sizeof (CameraFilesystemFile));
-	CBO(strlen(name), 
-	    sizeof(fs->folder[x].file[fs->folder[x].count - 1].name),
+	CBO(sizeof(fs->folder[x].file[fs->folder[x].count - 1].name),
+	    strlen(name),
 	    "append_file()");
 	strcpy (fs->folder[x].file[fs->folder[x].count - 1].name, name);
 	fs->folder[x].file[fs->folder[x].count - 1].info_dirty = 1;
@@ -492,8 +491,6 @@ append_file (CameraFilesystem *fs, int x, CameraFile *file, GPContext *context)
 	return (GP_OK);
 }
 
-static int gp_filesystem_lru_clear (CameraFilesystem *fs);
-
 /**
  * gp_filesystem_reset:
  * @fs: a #CameraFilesystem
@@ -573,7 +570,7 @@ gp_filesystem_folder_number (CameraFilesystem *fs, const char *folder,
 			     GPContext *context)
 {
 	int x, y, len;
-	char buf[128];
+	char buf[PATH_MAX];
 	CameraList *list;
 
 	CHECK_NULL (fs && folder);
@@ -726,6 +723,7 @@ delete_file (CameraFilesystem *fs, int x, int y)
 {
 	CameraFilesystemFile *new_fip;
 
+	gp_filesystem_lru_remove_one (fs, &fs->folder[x].file[y]);
 	/* Get rid of cached files */
 	if (fs->folder[x].file[y].preview) {
 		gp_file_unref (fs->folder[x].file[y].preview);
@@ -747,6 +745,10 @@ delete_file (CameraFilesystem *fs, int x, int y)
 		gp_file_unref (fs->folder[x].file[y].exif);
 		fs->folder[x].file[y].exif = NULL;
 	}
+	if (fs->folder[x].file[y].metadata) {
+		gp_file_unref (fs->folder[x].file[y].metadata);
+		fs->folder[x].file[y].metadata = NULL;
+	}
 
 	/* Move all files behind one position up */
 	if (y < fs->folder[x].count - 1)
@@ -927,7 +929,7 @@ gp_filesystem_list_folders (CameraFilesystem *fs, const char *folder,
 			    CameraList *list, GPContext *context)
 {
 	int x, y, j, offset, count;
-	char buf[1024];
+	char buf[PATH_MAX];
 	const char *name;
 	unsigned int len;
 
@@ -1538,6 +1540,11 @@ gp_filesystem_get_file_impl (CameraFilesystem *fs, const char *folder,
 			return (gp_file_copy (file,
 					fs->folder[x].file[y].exif));
 		break;
+	case GP_FILE_TYPE_METADATA:
+		if (fs->folder[x].file[y].metadata)
+			return (gp_file_copy (file,
+					fs->folder[x].file[y].metadata));
+		break;
 	default:
 		gp_context_error (context, _("Unknown file type %i."), type);
 		return (GP_ERROR);
@@ -1586,7 +1593,7 @@ gp_filesystem_get_file (CameraFilesystem *fs, const char *folder,
                         CameraFile *file, GPContext *context)
 {
 	int r;
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 	CameraFile *efile;
 	const char *data = NULL;
 	unsigned char *buf;
@@ -1605,7 +1612,7 @@ gp_filesystem_get_file (CameraFilesystem *fs, const char *folder,
 		 * Could not get preview (unsupported operation). Some 
 		 * cameras hide the thumbnail in EXIF data. Check it out.
 		 */
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 		GP_DEBUG ("Getting previews is not supported. Trying "
 			  "EXIF data...");
 		CR (gp_file_new (&efile));
@@ -1655,7 +1662,7 @@ gp_filesystem_get_file (CameraFilesystem *fs, const char *folder,
 		 * Some cameras hide EXIF data in thumbnails (!). Check it
 		 * out.
 		 */
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 		GP_DEBUG ("Getting EXIF data is not supported. Trying "
 			  "thumbnail...");
 		CR (gp_file_new (&efile));
@@ -1724,6 +1731,45 @@ gp_filesystem_set_info_funcs (CameraFilesystem *fs,
 	return (GP_OK);
 }
 
+/**
+ * gp_filesystem_set_funcs:
+ * @fs: a #CameraFilesystem
+ * @funcs: pointer to a struct of filesystem functions
+ * @data:
+ *
+ * Tells the filesystem which functions to call for camera/filesystem specific 
+ * functions.
+ *
+ * Return value: a gphoto2 error code.
+ **/
+int
+gp_filesystem_set_funcs	(CameraFilesystem *fs,
+			 CameraFilesystemFuncs *funcs,
+			 void *data)
+{
+	CHECK_NULL (fs);
+
+	fs->get_info_func	= funcs->get_info_func;
+	fs->set_info_func	= funcs->set_info_func;
+	fs->info_data = data;
+
+	fs->put_file_func	= funcs->put_file_func;
+	fs->delete_all_func	= funcs->delete_all_func;
+	fs->make_dir_func	= funcs->make_dir_func;
+	fs->remove_dir_func	= funcs->remove_dir_func;
+	fs->folder_data = data;
+
+	fs->file_list_func	= funcs->file_list_func;
+	fs->folder_list_func	= funcs->folder_list_func;
+	fs->list_data = data;
+
+	fs->delete_file_func	= funcs->del_file_func;
+	fs->get_file_func	= funcs->get_file_func;
+	fs->file_data = data;
+
+	return (GP_OK);
+}
+
 /**
  * gp_filesystem_get_info:
  * @fs: a #CameraFilesystem
@@ -1740,7 +1786,7 @@ gp_filesystem_get_info (CameraFilesystem *fs, const char *folder,
 			GPContext *context)
 {
 	int x, y;
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 	time_t t;
 #endif
 
@@ -1772,7 +1818,7 @@ gp_filesystem_get_info (CameraFilesystem *fs, const char *folder,
 	 * If we didn't get GP_FILE_INFO_MTIME, we'll have a look if we
 	 * can get it from EXIF data.
 	 */
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 	if (!(fs->folder[x].file[y].info.file.fields & GP_FILE_INFO_MTIME)) {
 		GP_DEBUG ("Did not get mtime. Trying EXIF information...");
 		t = gp_filesystem_get_exif_mtime (fs, folder, filename);
@@ -1907,119 +1953,22 @@ gp_filesystem_lru_free (CameraFilesystem *fs)
 	return (GP_OK);
 }
 
-/**
- * gp_get_free_memory:
- * @context: a #GPContext
- * @free:
- *
- * Reads the amount of free kB:
- *  - free memory + free swap for Linux
- *  XXX - free memory for BSD (need a way to find free swap)
- *
- * Return value: a gphoto2 error code.
- **/
 static int
-gp_get_free_memory (GPContext *context, unsigned *free)
+gp_filesystem_lru_count (CameraFilesystem *fs)
 {
-#ifdef HAVE_PROCMEMINFO
+	CameraFilesystemFile *ptr;
+	int count = 0;
 
-	char buf[1024], *head, *tail, *tmp;
-	int n, fd = -1;
-	*free=0;
-
-	if ((fd = open ("/proc/meminfo", O_RDONLY)) == -1) {
-		gp_context_error (context, _("Could not open '/proc/meminfo' "
-			"for reading ('%m'). Make sure the proc filesystem "
-			"is mounted."));
-		return (GP_ERROR);
+	if (!fs) return 0;
+	ptr = fs->lru_first;
+	while (ptr) {
+		if (ptr->normal || ptr->raw || ptr->audio)
+			count++;
+		ptr = ptr->lru_next;
 	}
-	
-	lseek (fd, 0L, SEEK_SET);
-	if ((n = read (fd, buf, sizeof (buf) - 1)) < 0) {
-		gp_context_error (context, _("An error occured while "
-			"reading '/proc/meminfo' ('%m')."));
-		return (GP_ERROR);
-	}
-	buf[n] = '\0';
-	n = 0;
-	head = buf;
-	do {
-		tail = strchr (head, ':');
-		if (!tail)
-			break;
-		*tail = '\0';
-		tmp = head;
-		head = tail + 1;
-		if (!strcmp (tmp, "MemFree")) {
-			*free += strtoul (head, NULL, 10);
-			n++;
-		} else if (!strcmp (tmp, "SwapFree")) {
-			*free += strtoul (head, NULL, 10);
-			n++;
-		}
-		tail = strchr (head, '\n');
-		if (!tail)
-			break;
-		head = tail + 1;
-	} while (n != 2);
-	return (GP_OK);
-
-#elif HAVE_SYSCTL && (__FreeBSD__ || __NetBSD__ || __OpenBSD__ || __APPLE__ )
-
-	int mib[2] = { CTL_HW, HW_PHYSMEM };
-	unsigned long value;
-	size_t valuelen = sizeof(value);
-	*free=0;
-	if (sysctl(mib, 2 , &value, &valuelen, NULL, 0) == -1) {
-		gp_context_error (context, _("sysctl call failed ('%m')."));
-		return (GP_ERROR);
-	}
-	*free=value;
-	return (GP_OK);
-
-#elif (defined(sun) || defined(__sun__)) && defined(__svr4__)
-
-	long freemem=0;
-	long freeswap=0;
-	int page_size;
-	static struct anoninfo anon;
-
-	*free = 0;
-
-	page_size = getpagesize();
-	if ( (freemem = sysconf(_SC_AVPHYS_PAGES)) == -1 )
-	  {
-	    gp_context_error (context, _("sysconf call failed ('%m')."));
-	    return (GP_ERROR);
-	  }
-
-	freemem *= page_size;
-
-	if (swapctl(SC_AINFO, &anon) == -1)
-	  {
-	    gp_context_error (context, _("swapctl call failed ('%m')."));
-	    return (GP_ERROR);
-	  }
-
-	freeswap = anon.ani_max - anon.ani_resv;
-	freeswap *= page_size;
-
-	printf("Free Memory : %ld Swap %ld\n",freemem,freeswap);
-
-	*free = freemem + freeswap;
-	return (GP_OK);
-
-#else
-
-	/* No way to know the free memory on this system */
-	*free = UINT_MAX;
-	return (GP_OK);
-
-#endif
-
+	return count;
 }
 
-
 static int
 gp_filesystem_lru_update (CameraFilesystem *fs, const char *folder,
 			  CameraFile *file, GPContext *context)
@@ -2029,7 +1978,7 @@ gp_filesystem_lru_update (CameraFilesystem *fs, const char *folder,
 	const char *filename;
 	unsigned long int size;
 	int x, y;
-	unsigned int free;
+	char cached_images[1024];
 
 	CHECK_NULL (fs && folder && file);
 
@@ -2038,25 +1987,34 @@ gp_filesystem_lru_update (CameraFilesystem *fs, const char *folder,
 	CR (gp_file_get_data_and_size (file, NULL, &size));
 
 	/*
-	 * The following is a simple case which is used to test the LRU.
-	 * I need to implement a way to pass a limit and then use it instead
-	 * of 600000. If the limit is not defined use the gp_get_free_memory
+	 * The following is a very simple case which is used to prune
+	 * the LRU. We keep PICTURES_TO_KEEP pictures in the LRU.
+	 *
+	 * We have 2 main scenarios:
+	 *	- query all thumbnails (repeatedly) ... they are cached and
+	 *	  are not pruned by lru free.
+	 *	- download all images, linear.	no real need for caching.
+	 *	- skip back 1 image (in viewers) (really? I don't know.)
+	 *
+	 * So lets just keep 2 pictures in memory.
 	 */
-#if 0
-	while (fs->lru_size + size > 600000) {
-		GP_DEBUG ("Freeing cached data before adding new data "
-			  "(cache=%ld, new=%ld)", fs->lru_size, size);
-		CR (gp_filesystem_lru_free (fs));
+	if (pictures_to_keep == -1) {
+		if (gp_setting_get ("libgphoto", "cached-images", cached_images) == GP_OK) {
+			pictures_to_keep = atoi(cached_images);
+		} else {
+			/* store a default setting */
+			sprintf (cached_images, "%d", PICTURES_TO_KEEP);
+			gp_setting_set ("libgphoto", "cached-images", cached_images);
+		}
 	}
-#endif
 
-	CR (gp_get_free_memory (context, &free));
-	while (free < (size / 1024 + 1024)) {
-		GP_DEBUG ("Freeing cached data before adding new data "
-			  "(cache=%ldB, new=%ldB, free=%dkB)",
-			  fs->lru_size, size, free);
+	if (pictures_to_keep < 0) /* also sanity check, but no upper limit. */
+		pictures_to_keep = PICTURES_TO_KEEP;
+
+	x = gp_filesystem_lru_count (fs);
+	while (x > pictures_to_keep) {
 		CR (gp_filesystem_lru_free (fs));
-		CR (gp_get_free_memory (context, &free));
+		x = gp_filesystem_lru_count (fs);
 	}
 
 	GP_DEBUG ("Adding file '%s' from folder '%s' to the fscache LRU list "
@@ -2084,6 +2042,7 @@ gp_filesystem_lru_update (CameraFilesystem *fs, const char *folder,
 			break;
 		case GP_FILE_TYPE_PREVIEW:
 		case GP_FILE_TYPE_EXIF:
+		case GP_FILE_TYPE_METADATA:
 			break;
 		default:
 			gp_context_error (context, _("Unknown file type %i."),
@@ -2176,7 +2135,7 @@ gp_filesystem_set_file_noop (CameraFilesystem *fs, const char *folder,
 	const char *filename;
 	int x, y, r;
 	time_t t;
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 	const char *data;
 	unsigned long int size;
 #endif
@@ -2238,6 +2197,12 @@ gp_filesystem_set_file_noop (CameraFilesystem *fs, const char *folder,
 		fs->folder[x].file[y].exif = file;
 		gp_file_ref (file);
 		break;
+	case GP_FILE_TYPE_METADATA:
+		if (fs->folder[x].file[y].metadata)
+			gp_file_unref (fs->folder[x].file[y].metadata);
+		fs->folder[x].file[y].metadata = file;
+		gp_file_ref (file);
+		break;
 	default:
 		gp_context_error (context, _("Unknown file type %i."), type);
 		return (GP_ERROR);
@@ -2260,7 +2225,7 @@ gp_filesystem_set_file_noop (CameraFilesystem *fs, const char *folder,
 	 * file, check if there is EXIF data in the file that contains
 	 * information on the mtime.
 	 */
-#ifdef HAVE_EXIF 
+#ifdef HAVE_LIBEXIF 
         if (!t && (type == GP_FILE_TYPE_NORMAL)) {
 		GP_DEBUG ("Searching data for mtime...");
 		CR (gp_file_get_data_and_size (file, &data, &size));
@@ -2272,7 +2237,7 @@ gp_filesystem_set_file_noop (CameraFilesystem *fs, const char *folder,
 	 * Still no mtime? Let's see if the camera offers us data of type
 	 * GP_FILE_TYPE_EXIF that includes information on the mtime.
 	 */
-#ifdef HAVE_EXIF
+#ifdef HAVE_LIBEXIF
 	if (!t) {
 		GP_DEBUG ("Trying EXIF information...");
 		t = gp_filesystem_get_exif_mtime (fs, folder, filename);
