@@ -28,9 +28,7 @@
 #include <stdio.h>
 #include <string.h>
 
-#ifdef HAVE_LTDL
 #include <ltdl.h>
-#endif
 
 #include "gphoto2-port-result.h"
 #include "gphoto2-port-info-list.h"
@@ -66,11 +64,7 @@ struct _GPPortPrivateCore {
 
 	GPPortInfo info;
 	GPPortOperations *ops;
-#ifdef HAVE_LTDL
 	lt_dlhandle lh;
-#else
-	void *lh; /* Library handle */
-#endif
 };
 
 /**
@@ -156,15 +150,10 @@ gp_port_set_info (GPPort *port, GPPortInfo info)
 		port->pc->ops = NULL;
 	}
 	if (port->pc->lh) {
-#ifdef HAVE_LTDL
 		lt_dlclose (port->pc->lh);
 		lt_dlexit ();
-#else
-		GP_SYSTEM_DLCLOSE (port->pc->lh);
-#endif
 	}
 
-#ifdef HAVE_LTDL
 	lt_dlinit ();
 	port->pc->lh = lt_dlopenext (info.library_filename);
 	if (!port->pc->lh) {
@@ -174,35 +163,15 @@ gp_port_set_info (GPPort *port, GPPortInfo info)
 		lt_dlexit ();
 		return (GP_ERROR_LIBRARY);
 	}
-#else
-	port->pc->lh = GP_SYSTEM_DLOPEN (info.library_filename);
-	if (!port->pc->lh) {
-		gp_log (GP_LOG_ERROR, "gphoto2-port", _("Could not load "
-			"'%s' ('%s')"), info.library_filename,
-			GP_SYSTEM_DLERROR ());
-		return (GP_ERROR_LIBRARY);
-	}
-#endif
 
 	/* Load the operations */
-#ifdef HAVE_LTDL
 	ops_func = lt_dlsym (port->pc->lh, "gp_port_library_operations");
-#else
-	ops_func = GP_SYSTEM_DLSYM (port->pc->lh, "gp_port_library_operations");
-#endif
 	if (!ops_func) {
-#ifdef HAVE_LTDL
 		gp_log (GP_LOG_ERROR, "gphoto2-port", _("Could not find "
 			"'gp_port_library_operations' in '%s' ('%s')"),
 			info.library_filename, lt_dlerror ());
 		lt_dlclose (port->pc->lh);
 		lt_dlexit ();
-#else
-		gp_log (GP_LOG_ERROR, "gphoto2-port", _("Could not find "
-			"'gp_port_library_operations' in '%s' ('%s')"),
-			info.library_filename, GP_SYSTEM_DLERROR ());
-		GP_SYSTEM_DLCLOSE (port->pc->lh);
-#endif
 		port->pc->lh = NULL;
 		return (GP_ERROR_LIBRARY);
 	}
@@ -230,6 +199,11 @@ gp_port_set_info (GPPort *port, GPPortInfo info)
 		port->settings.usb.altsetting = -1;
 		gp_port_set_timeout (port, 5000);
 		break;
+	case GP_PORT_DISK:
+		strncpy(port->settings.disk.mountpoint, 
+			strchr(info.path, ':') + 1, 
+			sizeof(port->settings.disk.mountpoint));
+		break;
 	default:
 		/* Nothing in here */
 		break;
@@ -333,12 +307,8 @@ gp_port_free (GPPort *port)
 		}
 
 		if (port->pc->lh) {
-#ifdef HAVE_LTDL
 			lt_dlclose (port->pc->lh);
 			lt_dlexit ();
-#else
-		        GP_SYSTEM_DLCLOSE (port->pc->lh);
-#endif
 			port->pc->lh = NULL;
 		}
 
@@ -816,6 +786,62 @@ gp_port_usb_msg_read (GPPort *port, int request, int value, int index,
         return (retval);
 }
 
+/* 
+ * The next two functions handle the request types 0x41 for write 
+ * and 0xc1 for read.
+ */
+
+int
+gp_port_usb_msg_interface_write (GPPort *port, int request, 
+	int value, int index, char *bytes, int size)
+{
+        int retval;
+
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", "Writing message "
+		"(request=0x%x value=0x%x index=0x%x size=%i=0x%x)...",
+		request, value, index, size, size);
+	gp_log_data ("gphoto2-port", bytes, size);
+
+	CHECK_NULL (port);
+	CHECK_INIT (port);
+
+	CHECK_SUPP (port, "msg_build", port->pc->ops->msg_interface_write);
+        retval = port->pc->ops->msg_interface_write(port, request, 
+        		value, index, bytes, size);
+	CHECK_RESULT (retval);
+
+        return (retval);
+}
+
+
+int
+gp_port_usb_msg_interface_read (GPPort *port, int request, int value, int index,
+	char *bytes, int size)
+{
+        int retval;
+
+	gp_log (GP_LOG_DEBUG, "gphoto2-port", "Reading message "
+		"(request=0x%x value=0x%x index=0x%x size=%i=0x%x)...",
+		request, value, index, size, size);
+
+	CHECK_NULL (port);
+	CHECK_INIT (port);
+
+	CHECK_SUPP (port, "msg_read", port->pc->ops->msg_interface_read);
+        retval = port->pc->ops->msg_interface_read (port, request, 
+        		value, index, bytes, size);
+	CHECK_RESULT (retval);
+
+	if (retval != size)
+		gp_log (GP_LOG_DEBUG, "gphoto2-port", "Could only read %i "
+			"out of %i byte(s)", retval, size);
+
+	gp_log_data ("gphoto2-port", bytes, retval);
+
+        return (retval);
+}
+
+
 /**
  * gp_port_set_error:
  * @port: a #GPPort
