@@ -72,17 +72,29 @@ struct _GPPortSettingsUSB {
 	int config;
 	int interface;
 	int altsetting;
+
+	/* must be last to avoid binary incompatibility.
+	 * luckily we just need to make sure this struct does not 
+	 * get larger than _GPPortSettingsSerial. */
+	char port[64];
+};
+
+typedef struct _GPPortSettingsDisk GPPortSettingsDisk;
+struct _GPPortSettingsDisk {
+	char mountpoint[128];
 };
 
 typedef union _GPPortSettings GPPortSettings;
 union _GPPortSettings {
 	GPPortSettingsSerial serial;
 	GPPortSettingsUSB usb;
+	GPPortSettingsDisk disk;
 };
 
 enum {
         GP_PORT_USB_ENDPOINT_IN,
-        GP_PORT_USB_ENDPOINT_OUT
+        GP_PORT_USB_ENDPOINT_OUT,
+        GP_PORT_USB_ENDPOINT_INT
 };
 
 typedef struct _GPPortPrivateLibrary GPPortPrivateLibrary;
@@ -152,9 +164,17 @@ int gp_port_usb_msg_write   (GPPort *port, int request, int value,
 			     int index, char *bytes, int size);
 int gp_port_usb_msg_read    (GPPort *port, int request, int value,
 			     int index, char *bytes, int size);
+int gp_port_usb_msg_interface_write    (GPPort *port, int request, 
+			    int value, int index, char *bytes, int size);
+int gp_port_usb_msg_interface_read    (GPPort *port, int request, 
+			    int value, int index, char *bytes, int size);
 
 /* Error reporting */
-int         gp_port_set_error (GPPort *port, const char *format, ...);
+int         gp_port_set_error (GPPort *port, const char *format, ...)
+#ifdef __GNUC__
+	__attribute__((__format__(printf,2,3)))
+#endif
+;
 const char *gp_port_get_error (GPPort *port);
 
 /* DEPRECATED */
