@@ -80,19 +80,27 @@ gp_port_library_list (GPPortInfoList *list)
 	GPPortInfo info;
 	struct usb_bus *bus;
 	struct usb_device *dev;
-	int nrofdevices = 0;
+	int nrofdevices = 0, i, i1, i2, unknownint;
 
 	/* default port first */
 	info.type = GP_PORT_USB;
 	strcpy (info.name, "Universal Serial Bus");
-        strcpy (info.path, "usb:");
+	strcpy (info.path, "usb:");
+	CHECK (gp_port_info_list_append (list, info));
+
+	/* generic matcher. This will catch passed XXX,YYY entries for instance. */
+	memset (info.name, 0, sizeof(info.name));
+	strcpy (info.path, "^usb:");
 	CHECK (gp_port_info_list_append (list, info));
 
 	usb_init ();
 	usb_find_busses ();
 	usb_find_devices ();
 
+	strcpy (info.name, "Universal Serial Bus");
+
 	bus = usb_get_busses();
+
 	/* Look and enumerate all USB ports. */
 	while (bus) {
 		for (dev = bus->devices; dev; dev = dev->next) {
@@ -103,6 +111,27 @@ gp_port_library_list (GPPortInfoList *list)
 				(dev->descriptor.bDeviceClass == USB_CLASS_COMM)
 			)
 				continue;
+			/* excepts HUBs, usually the interfaces have the classes, not
+			 * the device */
+			unknownint = 0;
+			for (i = 0; i < dev->descriptor.bNumConfigurations; i++) {
+				if (!dev->config) {
+					unknownint++;
+					continue;
+				}
+				for (i1 = 0; i1 < dev->config[i].bNumInterfaces; i1++)
+					for (i2 = 0; i2 < dev->config[i].interface[i1].num_altsetting; i2++) {
+						struct usb_interface_descriptor *intf = &dev->config[i].interface[i1].altsetting[i2]; 
+						if (	(intf->bInterfaceClass == USB_CLASS_HID)	||
+							(intf->bInterfaceClass == USB_CLASS_PRINTER)	||
+							(intf->bInterfaceClass == USB_CLASS_COMM))
+							continue;
+						unknownint++;
+					}
+			}
+			/* when we find only hids, printer or comm ifaces  ... skip this */
+			if (!unknownint)
+				continue;
 			/* Note: We do not skip USB storage. Some devices can support both,
 			 * and the Ricoh erronously reports it.
 			 */ 
@@ -130,6 +159,27 @@ gp_port_library_list (GPPortInfoList *list)
 				(dev->descriptor.bDeviceClass == USB_CLASS_COMM)
 			)
 				continue;
+			/* excepts HUBs, usually the interfaces have the classes, not
+			 * the device */
+			unknownint = 0;
+			for (i = 0; i < dev->descriptor.bNumConfigurations; i++) {
+				if (!dev->config) {
+					unknownint++;
+					continue;
+				}
+				for (i1 = 0; i1 < dev->config[i].bNumInterfaces; i1++)
+					for (i2 = 0; i2 < dev->config[i].interface[i1].num_altsetting; i2++) {
+						struct usb_interface_descriptor *intf = &dev->config[i].interface[i1].altsetting[i2]; 
+						if (	(intf->bInterfaceClass == USB_CLASS_HID)	||
+							(intf->bInterfaceClass == USB_CLASS_PRINTER)	||
+							(intf->bInterfaceClass == USB_CLASS_COMM))
+							continue;
+						unknownint++;
+					}
+			}
+			/* when we find only hids, printer or comm ifaces  ... skip this */
+			if (!unknownint)
+				continue;
 			/* Note: We do not skip USB storage. Some devices can support both,
 			 * and the Ricoh erronously reports it.
 			 */ 
@@ -343,6 +393,36 @@ gp_port_usb_msg_read_lib(GPPort *port, int request, int value, int index,
 		request, value, index, bytes, size, port->timeout);
 }
 
+/* The next two functions support the nonstandard request types 0x41 (write) 
+ * and 0xc1 (read), which are occasionally needed. 
+ */
+
+static int
+gp_port_usb_msg_interface_write_lib(GPPort *port, int request, 
+	int value, int index, char *bytes, int size)
+{
+	if (!port || !port->pl->dh)
+		return GP_ERROR_BAD_PARAMETERS;
+
+	return usb_control_msg(port->pl->dh, 
+		USB_TYPE_VENDOR | USB_RECIP_INTERFACE,
+		request, value, index, bytes, size, port->timeout);
+}
+
+
+static int
+gp_port_usb_msg_interface_read_lib(GPPort *port, int request, 
+	int value, int index, char *bytes, int size)
+{
+	if (!port || !port->pl->dh)
+		return GP_ERROR_BAD_PARAMETERS;
+
+	return usb_control_msg(port->pl->dh, 
+		USB_TYPE_VENDOR | USB_RECIP_INTERFACE | USB_ENDPOINT_IN,
+		request, value, index, bytes, size, port->timeout);
+}
+
+
 /*
  * This function applys changes to the device.
  *
@@ -734,6 +814,8 @@ gp_port_library_operations (void)
 	ops->clear_halt = gp_port_usb_clear_halt_lib;
 	ops->msg_write  = gp_port_usb_msg_write_lib;
 	ops->msg_read   = gp_port_usb_msg_read_lib;
+	ops->msg_interface_write  = gp_port_usb_msg_interface_write_lib;
+	ops->msg_interface_read   = gp_port_usb_msg_interface_read_lib;
 	ops->find_device = gp_port_usb_find_device_lib;
 	ops->find_device_by_class = gp_port_usb_find_device_by_class_lib;
 
